/*!
 * Quasar Framework v2.18.5
 * (c) 2015-present Razvan Stoenescu
 * Released under the MIT License.
 */

(()=>{window.Quasar.Lang.da={isoName:"da",nativeName:"Dansk",label:{clear:"Ryd",ok:"OK",cancel:"Annuller",close:"Luk",set:"S\xE6t",select:"V\xE6lg",reset:"Nulstil",remove:"Fjern",update:"Opdater",create:"Opret",search:"S\xF8g",filter:"Filtrer",refresh:"Opdater",expand:e=>e?`Udvid "${e}"`:"Udvide",collapse:e=>e?`Skjul "${e}"`:"Bryder sammen"},date:{days:"S\xF8ndag_Mandag_Tirsdag_Onsdag_Torsdag_Fredag_L\xF8rdag".split("_"),daysShort:"S\xF8n_Man_Tirs_Ons_Tors_Fre_L\xF8r".split("_"),months:"Januar_Februar_Marts_April_Maj_Juni_Juli_August_September_Oktober_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_Maj_Jun_Jul_Aug_Sep_Okt_Nov_Dec".split("_"),firstDayOfWeek:1,format24h:!0,pluralDay:"dage",prevMonth:"Forrige m\xE5ned",nextMonth:"N\xE6ste m\xE5ned",prevYear:"Forrige \xE5r",nextYear:"N\xE6ste \xE5r",today:"I dag",prevRangeYears:e=>`Forrige ${e} \xE5r`,nextRangeYears:e=>`N\xE6ste ${e} \xE5r`},table:{noData:"Ingen data tilg\xE6ngelig",noResults:"Ingen matchende resultater fundet",loading:"Indl\xE6ser...",selectedRecords:e=>e===1?"1 r\xE6kke valgt.":(e===0?"Ingen":e)+" r\xE6kker valgt.",recordsPerPage:"R\xE6kker per side:",allRows:"Alle",pagination:(e,r,t)=>e+"-"+r+" af "+t,columns:"Kolonner"},pagination:{first:"F\xF8rste side",prev:"Forrige side",next:"N\xE6ste side",last:"Sidste side"},editor:{url:"URL",bold:"Fed",italic:"Kursiv",strikethrough:"Gennemstreget",underline:"Understreget",unorderedList:"Punktliste",orderedList:"Numreret liste",subscript:"S\xE6nket",superscript:"H\xE6vet",hyperlink:"Hyperlink",toggleFullscreen:"Skift fuldsk\xE6rm",quote:"Citat",left:"Venstrejustering",center:"Centreret",right:"H\xF8jrejustering",justify:"Lige margener",print:"Udskriv",outdent:"Formindsk indrykning",indent:"For\xF8g indrykning",removeFormat:"Fjern formattering",formatting:"Formattering",fontSize:"Skriftst\xF8rrelse",align:"Justering",hr:"Inds\xE6t vandret streg",undo:"Fortryd",redo:"Gendan",heading1:"Overskrift 1",heading2:"Overskrift 2",heading3:"Overskrift 3",heading4:"Overskrift 4",heading5:"Overskrift 5",heading6:"Overskrift 6",paragraph:"Afsnit",code:"Kode",size1:"Meget lille",size2:"Lille",size3:"Normal",size4:"Mellemstor",size5:"Stor",size6:"Meget stor",size7:"St\xF8rst",defaultFont:"Standard skrifttype",viewSource:"Se kilde"},tree:{noNodes:"Ingen noder tilg\xE6ngelige",noResults:"Ingen matchende noder fundet"}};})();
