/*!
 * Quasar Framework v2.18.5
 * (c) 2015-present Razvan Stoenescu
 * Released under the MIT License.
 */

(()=>{window.Quasar.Lang.sl={isoName:"sl",nativeName:"Slovenski Jezik",label:{clear:"Po\u010Disti",ok:"V redu",cancel:"Prekli\u010Di",close:"Zapri",set:"Postavi",select:"Izberi",reset:"Obnovi",remove:"Odstrani",update:"Posodobi",create:"Ustvari",search:"I\u0161\u010Di",filter:"Filtriraj",refresh:"Ponovno nalo\u017Ei",expand:e=>e?`Raz\u0161iri "${e}"`:"Raz\u0161iri",collapse:e=>e?`Strni "${e}"`:"Strni"},date:{days:"Nedelja_Ponedeljek_Torek_Sreda_\u010Cetrtek_Petek_Sobota".split("_"),daysShort:"Ned_Pon_Tor_Sre_\u010Cet_Pet_Sob".split("_"),months:"Januar_Februar_Marec_April_Maj_Junij_Julij_Avgust_September_Oktober_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_Maj_Jun_Jul_Avg_Sep_Okt_Nov_Dec".split("_"),firstDayOfWeek:1,format24h:!0,pluralDay:"dni",prevMonth:"Prej\u0161nji mesec",nextMonth:"Naslednji mesec",prevYear:"Prej\u0161nje leto",nextYear:"Naslednje leto",today:"Danes",prevRangeYears:e=>`Prej\u0161nja ${e} leta`,nextRangeYears:e=>`Naslednja ${e} leta`},table:{noData:"Ni dosegljivih podatkov",noResults:"Ne najdem ustreznic",loading:"Nalagam...",selectedRecords:e=>e===1?"1 izbrana vrstica.":e===2?"2 izbrani vrstici.":(e===0?"Ni":e)+" izbranih vrstic.",recordsPerPage:"Vrstic na stran:",allRows:"Vse",pagination:(e,a,i)=>e+"-"+a+" od "+i,columns:"Stolpci"},pagination:{first:"Prva stran",prev:"Prej\u0161nja stran",next:"Naslednja stran",last:"Zadnja stran"},editor:{url:"URL",bold:"Krepko",italic:"Le\u017Ee\u010De",strikethrough:"Pre\u010Drtano",underline:"Pod\u010Drtaj",unorderedList:"Neo\u0161tevil\u010Den seznam",orderedList:"O\u0161tevil\u010Den seznam",subscript:"Podpisano",superscript:"Nadpisano",hyperlink:"Hiper povezava",toggleFullscreen:"Preklopi celoten zaslon",quote:"Citat",left:"Poravnaj levo",center:"Poravnaj na sredino",right:"Poravnaj desno",justify:"Obojestranska poravnava",print:"Natisni",outdent:"Zmanj\u0161aj zamik",indent:"Pove\u010Daj zamik",removeFormat:"Odstrani formatiranje",formatting:"Formatiranje",fontSize:"Velikost pisave",align:"Poravnava",hr:"Vstavi horizontalno pravilo",undo:"Razveljavi",redo:"Uveljavi",heading1:"Naslov 1",heading2:"Naslov 2",heading3:"Naslov 3",heading4:"Naslov 4",heading5:"Naslov 5",heading6:"Naslov 6",paragraph:"Odstavek",code:"Koda",size1:"Najmanj\u0161e",size2:"Manj\u0161e",size3:"Obi\u010Dajna velikost",size4:"Srednje veliko",size5:"Veliko",size6:"Ve\u010Dje",size7:"Najve\u010Dje",defaultFont:"Privzeta pisava",viewSource:"Prika\u017Ei vir"},tree:{noNodes:"Ni dosegljivih vozli\u0161\u010D",noResults:"Ne najdem ustreznih vozli\u0161\u010D"}};})();
