/*!
 * Quasar Framework v2.18.5
 * (c) 2015-present Razvan Stoenescu
 * Released under the MIT License.
 */

(()=>{window.Quasar.Lang.pl={isoName:"pl",nativeName:"Polski",label:{clear:"Wyczy\u015B\u0107",ok:"OK",cancel:"Anuluj",close:"Zamknij",set:"Ustaw",select:"Wybierz",reset:"Zresetuj",remove:"Usu\u0144",update:"Zaktualizuj",create:"Utw\xF3rz",search:"Szukaj",filter:"Filtruj",refresh:"Od\u015Bwie\u017C",expand:e=>e?`Rozwi\u0144 "${e}"`:"Zwi\u0119ksza\u0107",collapse:e=>e?`Zwi\u0144 "${e}"`:"Upadek"},date:{days:"Niedziela_Poniedzia\u0142ek_Wtorek_\u015Aroda_Czwartek_Pi\u0105tek_Sobota".split("_"),daysShort:"Nd_Pon_Wt_\u015Ar_Czw_Ptk_Sob".split("_"),months:"Stycze\u0144_Luty_Marzec_Kwiecie\u0144_Maj_Czerwiec_Lipiec_Sierpie\u0144_Wrzesie\u0144_Pa\u017Adziernik_Listopad_Grudzie\u0144".split("_"),monthsShort:"Sty_Lut_Mar_Kwi_Maj_Cze_Lip_Sie_Wrz_Pa\u017A_Lis_Gru".split("_"),firstDayOfWeek:1,format24h:!0,pluralDay:"dni",prevMonth:"Poprzedni miesi\u0105c",nextMonth:"Nast\u0119pny miesi\u0105c",prevYear:"Poprzedni rok",nextYear:"Nast\u0119pny rok",today:"Dzisiaj",prevRangeYears:e=>`Poprzedni ${e} lata`,nextRangeYears:e=>`Nast\u0119pnie ${e} lata`},table:{noData:"Brak dost\u0119pnych danych",noResults:"Nie znaleziono pasuj\u0105cych wpis\xF3w",loading:"\u0141adowanie...",selectedRecords:e=>e>1?e+" zaznaczony(ch) wiersz(y).":(e===0?"Brak":"1")+" zaznaczony wiersz.",recordsPerPage:"Wierszy na stron\u0119:",allRows:"Wszystkie",pagination:(e,a,i)=>e+"-"+a+" z "+i,columns:"Kolumny"},pagination:{first:"Pierwsza strona",prev:"Poprzednia strona",next:"Nast\u0119pna strona",last:"Ostatnia strona"},editor:{url:"URL",bold:"Pogrubienie",italic:"Kursywa",strikethrough:"Przekre\u015Blenie",underline:"Podkre\u015Blenie",unorderedList:"Nieuporz\u0105dkowana lista",orderedList:"Uporz\u0105dkowana lista",subscript:"Indeks dolny",superscript:"Indeks g\xF3rny",hyperlink:"Hiper\u0142\u0105cze",toggleFullscreen:"Prze\u0142\u0105cz na tryb pe\u0142noekranowy",quote:"Zacytuj",left:"Wyr\xF3wnaj do lewej",center:"Wyr\xF3wnaj do \u015Brodka",right:"Wyr\xF3wnaj do prawej",justify:"Wyjustuj",print:"Drukuj",outdent:"Zmniejsz wci\u0119cie",indent:"Zwi\u0119ksz wci\u0119cie",removeFormat:"Usu\u0144 formatowanie",formatting:"Formatowanie",fontSize:"Rozmiar czcionki",align:"Wyr\xF3wnanie",hr:"Wstaw poziom\u0105 lini\u0119",undo:"Cofnij",redo:"Przywr\xF3\u0107",heading1:"Nag\u0142\xF3wek 1",heading2:"Nag\u0142\xF3wek 2",heading3:"Nag\u0142\xF3wek 3",heading4:"Nag\u0142\xF3wek 4",heading5:"Nag\u0142\xF3wek 5",heading6:"Nag\u0142\xF3wek 6",paragraph:"Paragraf",code:"Kod",size1:"Bardzo ma\u0142a",size2:"Ma\u0142a",size3:"Normalna",size4:"\u015Arednio-du\u017Ca",size5:"Du\u017Ca",size6:"Bardzo du\u017Ca",size7:"Maksymalna",defaultFont:"Domy\u015Blna czcionka",viewSource:"Poka\u017C \u017Ar\xF3d\u0142o"},tree:{noNodes:"Brak dost\u0119pnych ga\u0142\u0119zi",noResults:"Nie znaleziono pasuj\u0105cych ga\u0142\u0119zi"}};})();
