/*!
 * Quasar Framework v2.18.5
 * (c) 2015-present Razvan Stoenescu
 * Released under the MIT License.
 */

(()=>{window.Quasar.Lang.et={isoName:"et",nativeName:"Eesti",label:{clear:"T\xFChjenda",ok:"OK",cancel:"T\xFChista",close:"Sulge",set:"M\xE4\xE4ra",select:"Vali",reset:"L\xE4htesta",remove:"Eemalda",update:"Uuenda",create:"Loo",search:"Otsi",filter:"Filtreeri",refresh:"V\xE4rskenda",expand:e=>e?`Laienda "${e}"`:"Laienda",collapse:e=>e?`Ahenda "${e}"`:"Ahenda"},date:{days:"p\xFChap\xE4ev_esmasp\xE4ev_teisip\xE4ev_kolmap\xE4ev_neljap\xE4ev_reede_laup\xE4ev".split("_"),daysShort:"P_E_T_K_N_R_L".split("_"),months:"jaanuar_veebruar_m\xE4rts_aprill_mai_juuni_juuli_august_september_oktoober_november_detsember".split("_"),monthsShort:"jaan_veebr_m\xE4rts_apr_mai_juuni_juuli_aug_sept_okt_nov_dets".split("_"),firstDayOfWeek:1,format24h:!0,pluralDay:"p\xE4eva",prevMonth:"Eelmine kuu",nextMonth:"J\xE4rgmisel kuul",prevYear:"Eelmine aasta",nextYear:"J\xE4rgmisel aastal",today:"T\xE4na",prevRangeYears:e=>`Eelmine ${e} aastad`,nextRangeYears:e=>`J\xE4rgmine ${e} aastat`},table:{noData:"Andmeid ei ole",noResults:"Sobivaid kirjeid ei leitud",loading:"Laadimine...",selectedRecords:e=>e===1?"1 kirje valitud.":e+" kirjet valitud.",recordsPerPage:"Kirjed lehel:",allRows:"K\xF5ik",pagination:(e,a,i)=>e+"-"+a+" / "+i,columns:"Veerud"},pagination:{first:"Esimene leht",prev:"Eelmine leht",next:"J\xE4rgmine leht",last:"Viimane leht"},editor:{url:"URL",bold:"Rasvane",italic:"Kaldkiri",strikethrough:"L\xE4bikriipsutatud",underline:"Allakriipsutatud",unorderedList:"J\xE4rjestamata loend",orderedList:"J\xE4rjestatud loend",subscript:"Alaindeks",superscript:"\xDClaindeks",hyperlink:"Link",toggleFullscreen:"T\xE4isekraan",quote:"Tsitaat",left:"Joonda vasakule",center:"Joonda keskele",right:"Joonda paremale",justify:"Joonda v\xF5rdselt",print:"Prindi",outdent:"V\xE4henda taanet",indent:"Suurenda taanet",removeFormat:"Eemalda vormindus",formatting:"Vormindamine",fontSize:"Fondi suurus",align:"Joonda",hr:"Horisontaalne joon",undo:"V\xF5ta tagasi",redo:"Tee uuesti",heading1:"Pealkiri 1",heading2:"Pealkiri 2",heading3:"Pealkiri 3",heading4:"Pealkiri 4",heading5:"Pealkiri 5",heading6:"Pealkiri 6",paragraph:"L\xF5ik",code:"Kood",size1:"V\xE4ga v\xE4ike",size2:"Natuke v\xE4ike",size3:"Normaalne",size4:"Keskmiselt suur",size5:"Suur",size6:"V\xE4ga suur",size7:"Maksimaalne",defaultFont:"Vaikefont",viewSource:"Kuva allikas"},tree:{noNodes:"\xDChtegi s\xF5lme pole saadaval",noResults:"Sobivaid s\xF5lmi ei leitud"}};})();
