/*!
 * Quasar Framework v2.18.5
 * (c) 2015-present Razvan Stoenescu
 * Released under the MIT License.
 */

(()=>{window.Quasar.Lang.hu={isoName:"hu",nativeName:"Magyar",label:{clear:"T\xF6rl\xE9s",ok:"OK",cancel:"M\xE9gsem",close:"Bez\xE1r\xE1s",set:"Be\xE1ll\xEDt\xE1s",select:"Kiv\xE1laszt\xE1s",reset:"Vissza\xE1ll\xEDt\xE1s",remove:"Elt\xE1vol\xEDt\xE1s",update:"M\xF3dos\xEDt\xE1s",create:"L\xE9trehoz\xE1s",search:"Keres\xE9s",filter:"Sz\u0171r\xE9s",refresh:"Friss\xEDt\xE9s",expand:e=>e?`A "${e}" kiterjeszt\xE9se`:"Kiterjed",collapse:e=>e?`A "${e}" \xF6sszecsuk\xE1sa`:"\xD6sszeoml\xE1s"},date:{days:"Vas\xE1rnap_H\xE9tf\u0151_Kedd_Szerda_Cs\xFCt\xF6rt\xF6k_P\xE9ntek_Szombat".split("_"),daysShort:"Vas_H\xE9t_Ke_Sze_Cs\xFC_P\xE9n_Szo".split("_"),months:"Janu\xE1r_Febru\xE1r_M\xE1rcius_\xC1prilis_M\xE1jus_J\xFAnius_J\xFAlius_Augusztus_Szeptember_Okt\xF3ber_November_December".split("_"),monthsShort:"Jan_Feb_M\xE1r_\xC1pr_M\xE1j_J\xFAn_J\xFAl_Aug_Szep_Okt_Nov_Dec".split("_"),firstDayOfWeek:1,format24h:!0,pluralDay:"nap",prevMonth:"El\u0151z\u0151 h\xF3nap",nextMonth:"K\xF6vetkez\u0151 h\xF3napban",prevYear:"El\u0151z\u0151 \xE9v",nextYear:"J\xF6v\u0151re",today:"Ma",prevRangeYears:e=>`El\u0151z\u0151 ${e} \xE9vek`,nextRangeYears:e=>`K\xF6vetkez\u0151 ${e} \xE9vek`},table:{noData:"Nincs el\xE9rhet\u0151 adat",noResults:"Nincsenek egyez\u0151 tal\xE1latok",loading:"Bet\xF6lt\xE9s...",selectedRecords:e=>e===1?"1 kiv\xE1lasztott elem.":(e===0?"Nincs":e)+" kiv\xE1lasztott elem.",recordsPerPage:"Elemek sz\xE1ma oldalank\xE9nt:",allRows:"\xD6sszes",pagination:(e,s,t)=>e+"-"+s+" / "+t,columns:"Oszlopok"},pagination:{first:"Els\u0151 oldal",prev:"El\u0151z\u0151 oldal",next:"K\xF6vetkez\u0151 oldal",last:"Utols\xF3 oldal"},editor:{url:"URL",bold:"F\xE9lk\xF6v\xE9r",italic:"D\u0151lt",strikethrough:"\xC1th\xFAzott",underline:"Al\xE1h\xFAzott",unorderedList:"Felsorol\xE1s",orderedList:"Sz\xE1moz\xE1s",subscript:"Als\xF3 index",superscript:"Fels\u0151 index",hyperlink:"Hivatkoz\xE1s",toggleFullscreen:"Teljes k\xE9perny\u0151",quote:"Id\xE9zet",left:"Balra igaz\xEDt\xE1s",center:"K\xF6z\xE9pre igaz\xEDt\xE1s",right:"Jobbra igaz\xEDt\xE1s",justify:"Sorkiz\xE1r\xE1s",print:"Nyomtat\xE1s",outdent:"Beh\xFAz\xE1s cs\xF6kkent\xE9se",indent:"Beh\xFAz\xE1s n\xF6vel\xE9se",removeFormat:"Form\xE1z\xE1s t\xF6rl\xE9se",formatting:"Form\xE1z\xE1s",fontSize:"Bet\u0171m\xE9ret",align:"Igaz\xEDt\xE1s",hr:"V\xEDzszintes elv\xE1laszt\xF3 beilleszt\xE9se",undo:"Visszavon\xE1s",redo:"M\xE9gis",heading1:"C\xEDmsor 1",heading2:"C\xEDmsor 2",heading3:"C\xEDmsor 3",heading4:"C\xEDmsor 4",heading5:"C\xEDmsor 5",heading6:"C\xEDmsor 6",paragraph:"Paragrafus",code:"K\xF3d",size1:"Nagyon kicsi",size2:"Kicsi",size3:"Norm\xE1l",size4:"K\xF6zepesen nagy",size5:"Nagy",size6:"Nagyon nagy",size7:"Maxim\xE1lis",defaultFont:"Alap\xE9rtelmezett bet\u0171t\xEDpus",viewSource:"Forr\xE1s megtekint\xE9se"},tree:{noNodes:"Nincsenek el\xE9rhet\u0151 elemek",noResults:"Nincsenek egyez\u0151 tal\xE1latok"}};})();
