/*!
 * Quasar Framework v2.18.5
 * (c) 2015-present Razvan Stoenescu
 * Released under the MIT License.
 */

(()=>{window.Quasar.Lang.pt={isoName:"pt",nativeName:"Portugu\xEAs",label:{clear:"Limpar",ok:"OK",cancel:"Cancelar",close:"Fechar",set:"Marcar",select:"Escolher",reset:"Limpar",remove:"Remover",update:"Atualizar",create:"Criar",search:"Procurar",filter:"Filtrar",refresh:"Recarregar",expand:e=>e?`Expandir "${e}"`:"Expandir",collapse:e=>e?`Recolher "${e}"`:"Colapso"},date:{days:"Domingo_Segunda-feira_Ter\xE7a-feira_Quarta-feira_Quinta-feira_Sexta-feira_S\xE1bado".split("_"),daysShort:"Dom_Seg_Ter_Qua_Qui_Sex_S\xE1b".split("_"),months:"Janeiro_Fevereiro_Mar\xE7o_Abril_Maio_Junho_Julho_Agosto_Setembro_Outubro_Novembro_Dezembro".split("_"),monthsShort:"Jan_Fev_Mar_Abr_Mai_Jun_Jul_Ago_Set_Out_Nov_Dez".split("_"),firstDayOfWeek:1,format24h:!0,pluralDay:"dias",prevMonth:"M\xEAs anterior",nextMonth:"Pr\xF3ximo m\xEAs",prevYear:"Ano anterior",nextYear:"Pr\xF3ximo ano",today:"Hoje",prevRangeYears:e=>`Anterior ${e} anos`,nextRangeYears:e=>`Pr\xF3ximo ${e} anos`},table:{noData:"Sem dados dispon\xEDveis",noResults:"N\xE3o foi encontrado nenhum resultado",loading:"A carregar...",selectedRecords:e=>e>0?e+" linha"+(e===1?" selecionada":"s selecionadas")+".":"Nenhuma linha selecionada.",recordsPerPage:"Linhas por p\xE1gina:",allRows:"Todas",pagination:(e,a,r)=>e+"-"+a+" de "+r,columns:"Colunas"},pagination:{first:"Primeira p\xE1gina",prev:"P\xE1gina anterior",next:"Pr\xF3xima p\xE1gina",last:"\xDAltima p\xE1gina"},editor:{url:"URL",bold:"Negrito",italic:"It\xE1lico",strikethrough:"Rasurado",underline:"Sublinhado",unorderedList:"Lista n\xE3o-ordenada",orderedList:"Lista ordenada",subscript:"Subscrito",superscript:"Sobrescrito",hyperlink:"Hyperlink",toggleFullscreen:"Maximizar",quote:"Cita\xE7\xE3o",left:"Alinhado \xE0 esquerda",center:"Alinhado ao centro",right:"Alinhado \xE0 direita",justify:"Justificado",print:"Imprimir",outdent:"Diminuir indenta\xE7\xE3o",indent:"Aumentar indenta\xE7\xE3o",removeFormat:"Remover formata\xE7\xE3o",formatting:"Formata\xE7\xE3o",fontSize:"Tamanho do tipo de letra",align:"Alinhar",hr:"Inserir linha horizontal",undo:"Desfazer",redo:"Refazer",heading1:"Cabe\xE7alho 1",heading2:"Cabe\xE7alho 2",heading3:"Cabe\xE7alho 3",heading4:"Cabe\xE7alho 4",heading5:"Cabe\xE7alho 5",heading6:"Cabe\xE7alho 6",paragraph:"Par\xE1grafo",code:"C\xF3digo",size1:"Muito pequeno",size2:"Pequeno",size3:"Normal",size4:"M\xE9dio",size5:"Grande",size6:"Enorme",size7:"M\xE1ximo",defaultFont:"Tipo de letra padr\xE3o",viewSource:"Exibir fonte"},tree:{noNodes:"Sem n\xF3s dispon\xEDveis",noResults:"Nenhum resultado encontrado"}};})();
