import{m as t}from"./index-BCkSasan.js";var i=class{constructor(t){this.init=t,this.records=this.init()}static{t(this,"ImperativeState")}reset(){this.records=this.init()}};export{i as s};
//# sourceMappingURL=chunk-VSLJSFIP-wJYFK5SS.js.map
