import{p as e}from"./treemap-KMMF4GRG-7ORZ52ND-Br-M8qul.js";import{m as r,p as a,a6 as t,W as s,a7 as n}from"./index-BCkSasan.js";import"./chunk-OMTJKCYW-B6LfB1Nw.js";var i={parse:r(async r=>{let t=await e("info",r);a.debug(t)},"parse")},o={version:n.version+""},d={parser:i,db:{getVersion:r(()=>o.version,"getVersion")},renderer:{draw:r((e,r,n)=>{a.debug("rendering info diagram\n"+e);let i=t(r);s(i,100,400,!0),i.append("g").append("text").attr("x",100).attr("y",40).attr("class","version").attr("font-size",32).style("text-anchor","middle").text(`v${n}`)},"draw")}};export{d as diagram};
//# sourceMappingURL=infoDiagram-JRAZ3BEV-CtisdNGY.js.map
