/*!
 * Quasar Framework v2.18.5
 * (c) 2015-present Razvan Stoenescu
 * Released under the MIT License.
 */

(()=>{window.Quasar.Lang.cs={isoName:"cs",nativeName:"\u010Ce\u0161tina",label:{clear:"Smazat",ok:"OK",cancel:"Zru\u0161it",close:"Zav\u0159\xEDt",set:"Nastavit",select:"Vybrat",reset:"Reset",remove:"Odebrat",update:"Opravit",create:"Vytvo\u0159it",search:"Hledat",filter:"Filtrovat",refresh:"Obnovit",expand:e=>e?`Rozbalit "${e}"`:"Roz\u0161\xED\u0159it",collapse:e=>e?`Sbalit "${e}"`:"Kolaps"},date:{days:"Ned\u011Ble_Pond\u011Bl\xED_\xDAter\xFD_St\u0159eda_\u010Ctvrtek_P\xE1tek_Sobota".split("_"),daysShort:"Ne_Po_\xDAt_St_\u010Ct_P\xE1_So".split("_"),months:"Leden_\xDAnor_B\u0159ezen_Duben_Kv\u011Bten_\u010Cerven_\u010Cervenec_Srpen_Z\xE1\u0159\xED_\u0158\xEDjen_Listopad_Prosinec".split("_"),monthsShort:"Led_\xDAno_B\u0159e_Dub_Kv\u011B_\u010Cvn_\u010Cvc_Srp_Z\xE1\u0159_\u0158\xEDj_Lis_Pro".split("_"),firstDayOfWeek:1,format24h:!0,pluralDay:"dny",prevMonth:'"P\u0159edchoz\xED m\u011Bs\xEDc',nextMonth:"P\u0159\xED\u0161t\xED m\u011Bs\xEDc",prevYear:"P\u0159edchoz\xED rok",nextYear:"P\u0159\xED\u0161t\xED rok",today:"Dnes",prevRangeYears:e=>`P\u0159edchoz\xED ${e} roky`,nextRangeYears:e=>`Dal\u0161\xED ${e} roky`},table:{noData:"\u017D\xE1dn\xE1 data k dispozici",noResults:"Nebyly nalezeny \u017E\xE1dn\xE9 odpov\xEDdaj\xEDc\xED z\xE1znamy",loading:"Na\u010D\xEDt\xE1 se...",selectedRecords:e=>{switch(e){case 0:return"Nejsou vybr\xE1ny \u017E\xE1dn\xE9 \u0159\xE1dky.";case 1:return"Vybr\xE1n 1 \u0159\xE1dek.";case 2:return"Vybr\xE1ny 2 \u0159\xE1dky.";case 3:return"Vybr\xE1ny 3 \u0159\xE1dky.";case 4:return"Vybr\xE1ny 4 \u0159\xE1dky."}return"Vybr\xE1no "+e+" \u0159\xE1dk\u016F."},recordsPerPage:"Po\u010Det \u0159\xE1dk\u016F na str\xE1nku:",allRows:"V\u0161echny",pagination:(e,t,a)=>e+"-"+t+" z "+a,columns:"Sloupce"},pagination:{first:"Prvn\xED str\xE1nka",prev:"P\u0159edchoz\xED str\xE1nka",next:"Dal\u0161\xED str\xE1nka",last:"Posledn\xED str\xE1nka"},editor:{url:"URL",bold:"Tu\u010Dn\u011B",italic:"Kurz\xEDva",strikethrough:"P\u0159e\u0161krtnut\xE9",underline:"Podtr\u017Een\xE9",unorderedList:"Odr\xE1\u017Ekov\xFD seznam",orderedList:"\u010C\xEDslovan\xFD seznam",subscript:"Doln\xED index",superscript:"Horn\xED index",hyperlink:"Odkaz",toggleFullscreen:"P\u0159epnout zobrazen\xED na celou obrazovku",quote:"Blokov\xE1 citace",left:"Zarovnat vlevo",center:"Zarovnat na st\u0159ed",right:"Zarovnat vpravo",justify:"Zarovnat do bloku",print:"Tisk",outdent:"Zmen\u0161it odsazen\xED",indent:"Zv\u011Bt\u0161it odsazen\xED",removeFormat:"Vymazat form\xE1tov\xE1n\xED",formatting:"Styl",fontSize:"Velikost p\xEDsma",align:"Zarovn\xE1n\xED",hr:"Vlo\u017Eit odd\u011Blova\u010D",undo:"Zp\u011Bt",redo:"Znovu",heading1:"Nadpis 1",heading2:"Nadpis 2",heading3:"Nadpis 3",heading4:"Nadpis 4",heading5:"Nadpis 5",heading6:"Nadpis 6",paragraph:"Odstavec",code:"K\xF3d",size1:"Velmi mal\xE9",size2:"Mal\xE9",size3:"Norm\xE1ln\xED",size4:"St\u0159edn\u011B velk\xE9",size5:"Velk\xE9",size6:"Velmi velk\xE9",size7:"Maximum",defaultFont:"V\xFDchoz\xED p\xEDsmo",viewSource:"Zobrazit zdroj"},tree:{noNodes:"\u017D\xE1dn\xE9 uzly",noResults:"\u017D\xE1dn\xE9 odpov\xEDdaj\xEDc\xED uzly nenalezeny"}};})();
