/*!
 * Quasar Framework v2.18.5
 * (c) 2015-present Razvan Stoenescu
 * Released under the MIT License.
 */

(()=>{window.Quasar.Lang.fi={isoName:"fi",nativeName:"Suomi",label:{clear:"Tyhjenn\xE4",ok:"OK",cancel:"Peruuta",close:"Sulje",set:"Aseta",select:"Valitse",reset:"Resetoi",remove:"Poista",update:"P\xE4ivit\xE4",create:"Luo",search:"Etsi",filter:"Suodata",refresh:"P\xE4ivit\xE4",expand:e=>e?`Laajenna "${e}"`:"Laajentaa",collapse:e=>e?`Tiivist\xE4 "${e}"`:"Romahdus"},date:{days:"sunnuntai_maanantai_tiistai_keskiviikko_torstai_perjantai_lauantai".split("_"),daysShort:"su_ma_ti_ke_to_pe_la".split("_"),months:"tammikuu_helmikuu_maaliskuu_huhtikuu_toukokuu_kes\xE4kuu_hein\xE4kuu_elokuu_syyskuu_lokakuu_marraskuu_joulukuu".split("_"),monthsShort:"tammi_helmi_maalis_huhti_touko_kes\xE4_hein\xE4_elo_syys_loka_marras_joulu".split("_"),firstDayOfWeek:1,format24h:!0,pluralDay:"p\xE4iv\xE4\xE4",prevMonth:"Edellinen kuukausi",nextMonth:"Ensi kuussa",prevYear:"Edellinen vuosi",nextYear:"Ensi vuonna",today:"T\xE4n\xE4\xE4n",prevRangeYears:e=>`Edellinen ${e} vuotta`,nextRangeYears:e=>`Seuraava ${e} vuotta`},pullToRefresh:{pull:"Ved\xE4 alas p\xE4ivitt\xE4\xE4ksesi",release:"Vapauta p\xE4ivitt\xE4\xE4ksesi",refresh:"P\xE4ivitet\xE4\xE4n..."},table:{noData:"Ei tietoja",noResults:"Ei tuloksia",loading:"Ladataan...",selectedRecords:e=>e===1?"1 rivi valittu.":e+" rivi\xE4 valittu.",recordsPerPage:"Rivej\xE4 sivulla:",allRows:"Kaikki",pagination:(e,i,t)=>e+"-"+i+" / "+t,columns:"Sarakkeet"},pagination:{first:"Ensimm\xE4inen sivu",prev:"Edellinen sivu",next:"Seuraava sivu",last:"Viimeinen sivu"},editor:{url:"URL",bold:"Lihavoitu",italic:"Kursivoitu",strikethrough:"Yliviivattu",underline:"Alleviivattu",unorderedList:"J\xE4rjest\xE4m\xE4t\xF6n Lista",orderedList:"J\xE4rjestetty Lista",subscript:"Alaindeksi",superscript:"Yl\xE4indeksi",hyperlink:"Hyperlinkki",toggleFullscreen:"Vaihda n\xE4ytt\xF6tilaa",quote:"Lainaus",left:"Asettele vasemmalle",center:"Asettele keskelle",right:"Asettele oikealle",justify:"Asettele tasaten",print:"Tulosta",outdent:"V\xE4henn\xE4 sisennyst\xE4",indent:"Kasvata sisennyst\xE4",removeFormat:"Poista muotoilu",formatting:"Muotoilu",fontSize:"Fonttikoko",align:"Asettelu",hr:"Lis\xE4\xE4 erotin",undo:"Kumoa",redo:"Toista",heading1:"Otsikko 1",heading2:"Otsikko 2",heading3:"Otsikko 3",heading4:"Otsikko 4",heading5:"Otsikko 5",heading6:"Otsikko 6",paragraph:"Kappale",code:"Koodi",size1:"Eritt\xE4in pieni",size2:"Pieni",size3:"Normaali",size4:"Isompi",size5:"Iso",size6:"Eritt\xE4in iso",size7:"Maksimi",defaultFont:"Oletusfontti",viewSource:"Katso l\xE4hde"},tree:{noNodes:"Ei solmuja saatavilla",noResults:"Ei vastaavia solmuja"}};})();
