import{V as r,Q as s,C as a,l as e}from"./chunk-BHQAJ5QI-YnFFpx6L.js";import"./chunk-ZZTKAOFA-DwR9LMCR.js";import"./chunk-6EQESGSB-Bvcr-WJf.js";import"./chunk-T244DUNM-k-VBWeHV.js";import{m as t}from"./index-BCkSasan.js";var o={parser:a,get db(){return new e},renderer:s,styles:r,init:t(r=>{r.class||(r.class={}),r.class.arrowMarkerAbsolute=r.arrowMarkerAbsolute},"init")};export{o as diagram};
//# sourceMappingURL=classDiagram-v2-KGF5QE6Q-Bo_VrUS6.js.map
