/*!
 * Quasar Framework v2.18.5
 * (c) 2015-present Razvan Stoenescu
 * Released under the MIT License.
 */

(()=>{window.Quasar.Lang.sr={isoName:"sr",nativeName:"srpski jezik",label:{clear:"Obri\u0161i",ok:"OK",cancel:"Odustani",close:"Zatvori",set:"Postavi",select:"Izaberi",reset:"Poni\u0161ti",remove:"Ukloni",update:"A\u017Euriraj",create:"Dodaj",search:"Tra\u017Ei",filter:"Filter",refresh:"Osve\u017Ei",expand:a=>a?`Pro\u0161iri "${a}"`:"Pro\u0161iriti",collapse:a=>a?`Skupi "${a}"`:"Skupiti"},date:{days:"Nedelja_Ponedeljak_Utorak_Sreda_\u010Cetvrtak_Petak_Subota".split("_"),daysShort:"Ned_Pon_Uto_Sre_\u010Cet_Pet_Sub".split("_"),months:"Januar_Februar_Mart_April_Maj_Jun_Jul_Avgust_Septembar_Oktobar_Novembar_Decembar".split("_"),monthsShort:"Jan_Feb_Mar_Apr_Maj_Jun_Jul_Avg_Sep_Okt_Nov_Dec".split("_"),firstDayOfWeek:1,format24h:!0,pluralDay:"dana",prevMonth:"Prethodni mesec",nextMonth:"Slede\u0107i mesec",prevYear:"Prethodna godina",nextYear:"Slede\u0107a godina",today:"Danas",prevRangeYears:a=>`\u041F\u0440\u0435\u0442\u0445\u043E\u0434\u043D\u0438 ${a} \u0433\u043E\u0434\u0438\u043D\u0435`,nextRangeYears:a=>`\u0421\u043B\u0435\u0434\u0435\u045B\u0435 ${a} \u0433\u043E\u0434\u0438\u043D\u0435`},table:{noData:"Nema podataka",noResults:"Nema odgovaraju\u0107ih zapisa",loading:"U\u010Ditavanje...",selectedRecords:a=>a>1?a+" izabranih redova.":(a===0?"Nema":"1")+" izabranih redova.",recordsPerPage:"Redova po stranici:",allRows:"Sve",pagination:(a,e,i)=>a+"-"+e+" od "+i,columns:"Kolone"},pagination:{first:"Prva stranica",prev:"Prethodna stranica",next:"Slede\u0107a stranica",last:"Poslednja stranica"},editor:{url:"URL",bold:"Podebljano",italic:"Nako\u0161eno",strikethrough:"Precrtano",underline:"Podcrtano",unorderedList:"Ozna\u010Dena lista",orderedList:"Numerisana lista",subscript:"Potpisano",superscript:"Natpisano",hyperlink:"Hiperlink",toggleFullscreen:"Ceo ekran",quote:"Citat",left:"Poravnati ulevo",center:"Centrirati",right:"Poravnati udesno",justify:"Poravnati obostrano",print:"\u0160tampa",outdent:"Smanjiti uvla\u010Denje",indent:"Pove\u0107ati uvla\u010Denje",removeFormat:"Ukloniti formatiranje",formatting:"Formatirati",fontSize:"Veli\u010Dina slova",align:"Poravnati",hr:"Ubaciti horizontalnu liniju",undo:"Poni\u0161titi",redo:"Vratiti",heading1:"Naslov 1",heading2:"Naslov 2",heading3:"Naslov 3",heading4:"Naslov 4",heading5:"Naslov 5",heading6:"Naslov 6",paragraph:"Paragraf",code:"Kod",size1:"Najmanje",size2:"Manje",size3:"Normalno",size4:"Srednje Veliko",size5:"Veliko",size6:"Ve\u0107e",size7:"Najve\u0107e",defaultFont:"Podrazumevani font",viewSource:"Pogledaj izvor"},tree:{noNodes:"Nema nijednog \u010Dvora",noResults:"Nema odgovaraju\u0107ih \u010Dvorova"}};})();
