import{m as t,ao as r,q as a,f as e}from"./index-BCkSasan.js";var s=a(e()),l=t((t,r)=>{let a=t.append("rect");if(a.attr("x",r.x),a.attr("y",r.y),a.attr("fill",r.fill),a.attr("stroke",r.stroke),a.attr("width",r.width),a.attr("height",r.height),r.name&&a.attr("name",r.name),r.rx&&a.attr("rx",r.rx),r.ry&&a.attr("ry",r.ry),void 0!==r.attrs)for(let t in r.attrs)a.attr(t,r.attrs[t]);return r.class&&a.attr("class",r.class),a},"drawRect"),x=t((t,r)=>{let a={x:r.startx,y:r.starty,width:r.stopx-r.startx,height:r.stopy-r.starty,fill:r.fill,stroke:r.stroke,class:"rect"};l(t,a).lower()},"drawBackgroundRect"),i=t((t,a)=>{let e=a.text.replace(r," "),s=t.append("text");s.attr("x",a.x),s.attr("y",a.y),s.attr("class","legend"),s.style("text-anchor",a.anchor),a.class&&s.attr("class",a.class);let l=s.append("tspan");return l.attr("x",a.x+2*a.textMargin),l.text(e),s},"drawText"),n=t((t,r,a,e)=>{let l=t.append("image");l.attr("x",r),l.attr("y",a);let x=(0,s.sanitizeUrl)(e);l.attr("xlink:href",x)},"drawImage"),d=t((t,r,a,e)=>{let l=t.append("use");l.attr("x",r),l.attr("y",a);let x=(0,s.sanitizeUrl)(e);l.attr("xlink:href",`#${x}`)},"drawEmbeddedImage"),h=t(()=>({x:0,y:0,width:100,height:100,fill:"#EDF2AE",stroke:"#666",anchor:"start",rx:0,ry:0}),"getNoteRect"),o=t(()=>({x:0,y:0,width:100,height:100,"text-anchor":"start",style:"#666",textMargin:0,rx:0,ry:0,tspan:!0}),"getTextObj");export{d as E,n as d,o as f,x as g,h,l as x,i as y};
//# sourceMappingURL=chunk-WONJCSFZ-CDkYLZk3.js.map
