import{m as c}from"./index-BCkSasan.js";function t(c,t){c.accDescr&&t.setAccDescription?.(c.accDescr),c.accTitle&&t.setAccTitle?.(c.accTitle),c.title&&t.setDiagramTitle?.(c.title)}c(t,"populateCommonDb");export{t as c};
//# sourceMappingURL=chunk-4KE642ED-DDvPuFrd.js.map
