/*!
 * Quasar Framework v2.18.5
 * (c) 2015-present Razvan Stoenescu
 * Released under the MIT License.
 */

(()=>{window.Quasar.Lang.hr={isoName:"hr",nativeName:"Hrvatski jezik",label:{clear:"O\u010Disti",ok:"OK",cancel:"Odustani",close:"Zatvori",set:"Postavi",select:"Izaberi",reset:"Poni\u0161ti",remove:"Ukloni",update:"A\u017Euriraj",create:"Dodaj",search:"Tra\u017Ei",filter:"Filter",refresh:"Osvje\u017Ei",expand:a=>a?`Pro\u0161iri "${a}"`:"Pro\u0161iriti",collapse:a=>a?`Sa\u017Emi "${a}"`:"Kolaps"},date:{days:"Nedjelja_Ponedjeljak_Utorak_Srijeda_\u010Cetvrtak_Petak_Subota".split("_"),daysShort:"Ned_Pon_Uto_Sri_\u010Cet_Pet_Sub".split("_"),months:"Sije\u010Danj_Velja\u010Da_O\u017Eujak_Travanj_Svibanj_Lipanj_Srpanj_Kolovoz_Rujan_Listopad_Studeni_Prosinac".split("_"),monthsShort:"Sij_Velj_O\u017Eu_Tra_Svi_Lip_Srp_Kol_Ruj_Lis_Stu_Pro".split("_"),firstDayOfWeek:1,format24h:!0,pluralDay:"dana",prevMonth:"Prethodni mjesec",nextMonth:"Sljede\u0107i mjesec",prevYear:"Prethodna godina",nextYear:"Sljede\u0107e godine",today:"Danas",prevRangeYears:a=>`Prethodno ${a} godine`,nextRangeYears:a=>`Sljede\u0107e ${a} godine`},table:{noData:"Nema podataka",noResults:"Nema odgovaraju\u0107ih zapisa",loading:"U\u010Ditavanje...",selectedRecords:a=>a>1?a+" izabranih redova.":(a===0?"Nema":"1")+" izabranih redova.",recordsPerPage:"Redova po stranici:",allRows:"Sve",pagination:(a,e,i)=>a+"-"+e+" od "+i,columns:"Stupci"},pagination:{first:"Prva stranica",prev:"Prethodna stranica",next:"Sljede\u0107a stranica",last:"Posljednja stranica"},editor:{url:"URL",bold:"Podebljano",italic:"Nako\u0161eno",strikethrough:"Precrtano",underline:"Podcrtano",unorderedList:"Ozna\u010Dena lista",orderedList:"Numerirana lista",subscript:"Potpisano",superscript:"Natpisano",hyperlink:"Hiperlink",toggleFullscreen:"Puni zaslon",quote:"Citat",left:"Poravnati ulijevo",center:"Centrirati",right:"Poravnati udesno",justify:"Poravnati obostrano",print:"Ispis",outdent:"Smanjiti uvla\u010Denje",indent:"Pove\u0107ati uvla\u010Denje",removeFormat:"Ukloniti formatiranje",formatting:"Formatirati",fontSize:"Veli\u010Dina slova",align:"Poravnati",hr:"Ubaciti vodoravno ravnalo",undo:"Poni\u0161titi",redo:"Vratiti",heading1:"Naslov 1",heading2:"Naslov 2",heading3:"Naslov 3",heading4:"Naslov 4",heading5:"Naslov 5",heading6:"Naslov 6",paragraph:"Odlomak",code:"Kod",size1:"Vrlo malo",size2:"Malo manje",size3:"Normalno",size4:"Srednje",size5:"Veliko",size6:"Vrlo veliko",size7:"Nejve\u0107e",defaultFont:"Zadani font",viewSource:"Pogledaj izvor"},tree:{noNodes:"Nema nijednog \u010Dvora",noResults:"Nema odgovaraju\u0107ih \u010Dvorova"}};})();
