import{m as e,i as o}from"./index-BCkSasan.js";var t=e((e,t)=>{let n;return"sandbox"===t&&(n=o("#i"+e)),o("sandbox"===t?n.nodes()[0].contentDocument.body:"body").select(`[id="${e}"]`)},"getDiagramElement");export{t as m};
//# sourceMappingURL=chunk-6EQESGSB-Bvcr-WJf.js.map
