/*!
 * Quasar Framework v2.18.5
 * (c) 2015-present Razvan Stoenescu
 * Released under the MIT License.
 */

(()=>{window.Quasar.Lang.koKR={isoName:"ko-KR",nativeName:"\uD55C\uAD6D\uC5B4",label:{clear:"\uCD08\uAE30\uD654",ok:"\uD655\uC778",cancel:"\uCDE8\uC18C",close:"\uB2EB\uAE30",set:"\uC124\uC815",select:"\uC120\uD0DD",reset:"\uCD08\uAE30\uD654",remove:"\uC0AD\uC81C",update:"\uC5C5\uB370\uC774\uD2B8",create:"\uC0DD\uC131",search:"\uAC80\uC0C9",filter:"\uD544\uD130",refresh:"\uC0C8\uB85C \uACE0\uCE68",expand:e=>e?`"${e}" \uD655\uC7A5`:"\uD655\uC7A5\uD558\uB2E4",collapse:e=>e?`"${e}" \uC811\uAE30`:"\uBB34\uB108\uC9C0\uB2E4"},date:{days:"\uC77C\uC694\uC77C_\uC6D4\uC694\uC77C_\uD654\uC694\uC77C_\uC218\uC694\uC77C_\uBAA9\uC694\uC77C_\uAE08\uC694\uC77C_\uD1A0\uC694\uC77C".split("_"),daysShort:"\uC77C_\uC6D4_\uD654_\uC218_\uBAA9_\uAE08_\uD1A0".split("_"),months:"1\uC6D4_2\uC6D4_3\uC6D4_4\uC6D4_5\uC6D4_6\uC6D4_7\uC6D4_8\uC6D4_9\uC6D4_10\uC6D4_11\uC6D4_12\uC6D4".split("_"),monthsShort:"1\uC6D4_2\uC6D4_3\uC6D4_4\uC6D4_5\uC6D4_6\uC6D4_7\uC6D4_8\uC6D4_9\uC6D4_10\uC6D4_11\uC6D4_12\uC6D4".split("_"),firstDayOfWeek:0,format24h:!1,pluralDay:"\uC77C",prevMonth:"\uC9C0\uB09C \uB2EC",nextMonth:"\uB2E4\uC74C \uB2EC",prevYear:"\uC791\uB144",nextYear:"\uB0B4\uB144",today:"\uC624\uB298",prevRangeYears:e=>`\uC774\uC804 ${e} \uB144`,nextRangeYears:e=>`\uB2E4\uC74C ${e} \uB144`},table:{noData:"\uB370\uC774\uD130\uAC00 \uC5C6\uC2B5\uB2C8\uB2E4.",noResults:"\uACB0\uACFC\uAC00 \uC5C6\uC2B5\uB2C8\uB2E4.",loading:"\uB85C\uB4DC \uC911...",selectedRecords:e=>e>0?e+" \uAC1C\uAC00 \uC120\uD0DD \uB418\uC5C8\uC2B5\uB2C8\uB2E4.":"\uC120\uD0DD\uB41C \uD56D\uBAA9\uC774 \uC5C6\uC2B5\uB2C8\uB2E4.",recordsPerPage:"\uD398\uC774\uC9C0 \uB2F9 \uAC1C\uC218:",allRows:"\uC804\uCCB4",pagination:(e,t,a)=>a+" \uC911 "+e+"-"+t,columns:"\uC5F4"},pagination:{first:"\uCCAB \uD398\uC774\uC9C0",prev:"\uC774\uC804 \uD398\uC774\uC9C0",next:"\uB2E4\uC74C \uD398\uC774\uC9C0",last:"\uB9C8\uC9C0\uB9C9 \uD398\uC774\uC9C0"},editor:{url:"URL",bold:"\uAD75\uAC8C",italic:"\uAE30\uC6B8\uC774\uAE30",strikethrough:"\uCDE8\uC18C\uC120",underline:"\uBC11\uC904",unorderedList:"\uBE44\uC21C\uCC28 \uBAA9\uB85D",orderedList:"\uC21C\uC11C \uBAA9\uB85D",subscript:"\uC544\uB798 \uCCA8\uC790",superscript:"\uC704 \uCCA8\uC790",hyperlink:"\uD558\uC774\uD37C\uB9C1\uD06C",toggleFullscreen:"\uC804\uCCB4 \uD654\uBA74",quote:"\uB530\uC634\uD45C",left:"\uC67C\uCABD \uC815\uB82C",center:"\uAC00\uC6B4\uB370 \uC815\uB82C",right:"\uC624\uB978\uCABD \uC815\uB82C",justify:"\uC138\uB85C \uC815\uB82C",print:"\uCD9C\uB825",outdent:"\uB4E4\uC5EC \uC4F0\uAE30",indent:"\uB0B4\uC5B4 \uC4F0\uAE30",removeFormat:"\uD3EC\uB9F7\uD305 \uC81C\uAC70",formatting:"\uD3EC\uB9F7\uD305",fontSize:"\uAE00\uAF34 \uD06C\uAE30",align:"\uAC00\uB85C \uC815\uB82C",hr:"\uAC00\uB85C\uC904 \uB123\uAE30",undo:"\uC2E4\uD589\uCDE8\uC18C",redo:"\uB2E4\uC2DC\uD558\uAE30",heading1:"\uC81C\uBAA9 1",heading2:"\uC81C\uBAA9 2",heading3:"\uC81C\uBAA9 3",heading4:"\uC81C\uBAA9 4",heading5:"\uC81C\uBAA9 5",heading6:"\uC81C\uBAA9 6",paragraph:"\uB2E8\uB77D",code:"\uCF54\uB4DC",size1:"\uB9E4\uC6B0 \uC791\uAC8C",size2:"\uC791\uAC8C",size3:"\uBCF4\uD1B5",size4:"\uC57D\uAC04 \uD06C\uAC8C",size5:"\uD06C\uAC8C",size6:"\uC544\uC8FC \uD06C\uAC8C",size7:"\uCD5C\uB300",defaultFont:"\uAE30\uBCF8 \uAE00\uAF34",viewSource:"\uC18C\uC2A4\uBCF4\uAE30"},tree:{noNodes:"\uAC00\uB2A5\uD55C \uD56D\uBAA9\uC774 \uC5C6\uC2B5\uB2C8\uB2E4",noResults:"\uD56D\uBAA9\uC744 \uCC3E\uC744 \uC218 \uC5C6\uC2B5\uB2C8\uB2E4"}};})();
