/*!
 * Quasar Framework v2.18.5
 * (c) 2015-present Razvan Stoenescu
 * Released under the MIT License.
 */

(()=>{window.Quasar.Lang.azLatn={isoName:"az-Latn",nativeName:"Azerbaijani (latin)",label:{clear:"T\u0259mizl\u0259",ok:"OK",cancel:"L\u0259\u011Fv et",close:"Ba\u011Fla",set:"Ayarla",select:"Se\xE7",reset:"S\u0131f\u0131rla",remove:"Sil",update:"G\xFCnc\u0259ll\u0259",create:"Yarat",search:"Axtar",filter:"Filtrl\u0259",refresh:"Yenil\u0259",expand:a=>a?`"${a}" geni\u015Fl\u0259ndirin`:"Geni\u015Fl\u0259ndirin",collapse:a=>a?`"${a}" y\u0131\u011Fcamla\u015Fd\u0131r\u0131n`:"Y\u0131x\u0131lma"},date:{days:"Bazar_Bazar ert\u0259si_\xC7\u0259r\u015F\u0259nb\u0259 ax\u015Fam\u0131_\xC7\u0259r\u015F\u0259nb\u0259_C\xFCm\u0259 ax\u015Fam\u0131_C\xFCm\u0259_\u015E\u0259nb\u0259".split("_"),daysShort:"B_B.E_\xC7_\xC7.E_C.A_C_\u015E".split("_"),months:"Yanvar_Fevral_Mart_Aprel_May_\u0130yun_\u0130yul_Avqust_Sentyabr_Oktyabr_Noyabr_Dekabr".split("_"),monthsShort:"Yan_Fev_Mar_Apr_May_\u0130yn_\u0130yl_Avq_Sen_Okt_Noy_Dek".split("_"),firstDayOfWeek:1,format24h:!0,pluralDay:"g\xFCn",prevMonth:"\xD6t\u0259n ay",nextMonth:"G\u0259l\u0259n ay",prevYear:"\xD6t\u0259n il",nextYear:"G\u0259l\u0259n il",today:"Bu g\xFCn",prevRangeYears:a=>`\u018Fvv\u0259lki ${a} ill\u0259r`,nextRangeYears:a=>`Ard\u0131 ${a} ill\u0259r`},table:{noData:"M\u0259lumat yoxdur",noResults:"Uy\u011Fun m\u0259lumat tap\u0131lmad\u0131",loading:"Y\xFCkl\u0259nir...",selectedRecords:a=>a===1?"1 record selected.":(a===0?"No":a)+" se\xE7ilmi\u015F m\u0259lumat.",recordsPerPage:"H\u0259r s\u0259hif\u0259d\u0259ki m\u0259lumat:",allRows:"B\xFCt\xFCn",pagination:(a,e,r)=>a+"-"+e+" c\u0259mi "+r,columns:"S\xFCtun"},pagination:{first:"\u0130lk s\u0259hif\u0259",prev:"\u018Fvv\u0259lki s\u0259hif\u0259",next:"N\xF6vb\u0259ti s\u0259hif\u0259",last:"Son s\u0259hif\u0259"},editor:{url:"URL",bold:"Bold",italic:"Italic",strikethrough:"Strikethrough",underline:"Altdan x\u0259tt",unorderedList:"S\u0131ralanmam\u0131\u015F siyah\u0131",orderedList:"S\u0131ralanm\u0131\u015F siyah\u0131",subscript:"Alt yaz\u0131",superscript:"\xDCst yaz\u0131",hyperlink:"Link",toggleFullscreen:"Tam ekran\u0131 a\xE7/ba\u011Fla",quote:"Sitat",left:"Sol t\u0259r\u0259f",center:"Orta",right:"Ra\u011F t\u0259r\u0259f",justify:"M\u0259rk\u0259zl\u0259\u015Fdir",print:"\xC7ap et",outdent:"Abzas\u0131 azalt",indent:"Abzas\u0131 art\u0131r",removeFormat:"Formatlaman\u0131 sil",formatting:"Formatlama",fontSize:"Font \xF6l\xE7\xFCs\xFC",align:"T\u0259r\u0259f",hr:"Horizantal x\u0259tt \u0259lav\u0259 et",undo:"Geri qaytar",redo:"T\u0259krarla",heading1:"Ba\u015Fl\u0131q 1",heading2:"Ba\u015Fl\u0131q 2",heading3:"Ba\u015Fl\u0131q 3",heading4:"Ba\u015Fl\u0131q 4",heading5:"Ba\u015Fl\u0131q 5",heading6:"Ba\u015Fl\u0131q 6",paragraph:"Paraqraf",code:"Kod",size1:"\xC7ox ki\xE7ik",size2:"Bir az ki\xE7ik",size3:"Normal",size4:"Orta-b\xF6y\xFCk",size5:"B\xF6y\xFCk",size6:"\xC7ox b\xF6y\xFCk",size7:"Maksimum",defaultFont:"Varsay\u0131lan font",viewSource:"Qayna\u011F\u0131 g\xF6r"},tree:{noNodes:"D\xFCy\xFCn yoxdur",noResults:"Uy\u011Fun d\xFCy\xFCn yoxdur"}};})();
