/*!
 * Quasar Framework v2.18.5
 * (c) 2015-present Razvan Stoenescu
 * Released under the MIT License.
 */

(()=>{window.Quasar.Lang.sm={isoName:"sm",nativeName:"Fa'as\u0101moa",label:{clear:"Kilia",ok:"'Oukei",cancel:"Fa'al\u0113aog\u0101ina",close:"Tapuni",set:"Seti",select:"Filifili",reset:"Toe seti",remove:"'Ave'ese",update:"Fa'afou",create:"Fatu",search:"S\u0101'ili",filter:"Fa'amam\u0101",refresh:"Fa'afou",expand:a=>a?`Fa'alautele "${a}"`:"Faalautele",collapse:a=>a?`Pa'u "${a}"`:"Pa'u"},date:{days:"Aso S\u0101_Aso Gafua_Aso Lua_Aso Lulu_Aso Tofi_Aso Faraile_Aso To'onai".split("_"),daysShort:"S\u0101_Gaf_Lua_Lul_Tof_Far_Too".split("_"),months:"Ianuari_Fepuari_Mati_'Aperila_M\u0113_Iuni_Iulai_'Aokuso_S\u0113tema_'Oketopa_N\u014Dvema_T\u0113sema".split("_"),monthsShort:"Ian_Fep_Mat_Ape_M\u0113_Iun_Iul_Auk_S\u0113t_Oke_N\u014Dv_T\u0113s".split("_"),firstDayOfWeek:1,format24h:!1,pluralDay:"aso",prevMonth:"Masina Mai le masina talu ai",nextMonth:"Le masina fou",prevYear:"Tausaga talu ai",nextYear:"I Le Vafuapalo",today:"Le as\u014D",prevRangeYears:a=>`Talu ai ${a} tausaga`,nextRangeYears:a=>`Le isi ${a} tausaga`},table:{noData:"L\u0113ai ni fa'amatalaga o maua",noResults:"L\u0113ai ni fa'amaumauga na maua",loading:"'\u0100matalia...",selectedRecords:a=>a===1?"1 le laina 'ua filifilia.":(a===0?"L\u0113ai ni":a)+" laina 'ua filifilia.",recordsPerPage:"Laina 'i le \u012Bt\u016Blau:",allRows:"Laina 'uma",pagination:(a,i,e)=>a+"-"+i+" o "+e,columns:"Pout\u016B"},pagination:{first:"Itulau muamua",prev:"Itulau muamua",next:"Isi Itulau",last:"Itulau mulimuli"},editor:{url:"Tu\u0101tusi initaneti",bold:"Fa'aolaola",italic:"Fa'api'o",strikethrough:"Kolosi",underline:"Vase lalo",unorderedList:"Lisi 'e fe'i fa'atulagaina",orderedList:"Lisi 'ua fa'atulagaina",subscript:"Fa'anini'i i lalo",superscript:"Fa'anini'i i luga",hyperlink:"So'otaga initaneti",toggleFullscreen:"Fa'atel\u0113 ma fa'alititi le va'ai",quote:"Saunoaga",left:"\u014Cgatasi agavale",center:"\u014Cgatasi 'og\u0101totonu",right:"\u014Cgatasi taumatau",justify:"\u014Cgatasi lautele",print:"Lolomi",outdent:"Fa'alaititi le va",indent:"Fa'atel\u0113 le va",removeFormat:"'Ave'ese le fa'atulagaga",formatting:"Fa'atulagaga",fontSize:"Tel\u0113 o le mata'itusi",align:"\u014Cgatasi",hr:"Fa'aopoopo se laina",undo:"Toe 'ave'ese",redo:"Toe fai",heading1:"Ulutala 1",heading2:"Ulutala 2",heading3:"Ulutala 3",heading4:"Ulutala 4",heading5:"Ulutala 5",heading6:"Ulutala 6",paragraph:"Palakalafa",code:"Fa'ailo fa'akomepiuta",size1:"Matu\u0101 laitiiti",size2:"Fa'alaititi",size3:"T\u016Blaga masani",size4:"L\u0101po'a feololo",size5:"L\u0101po'a",size6:"L\u0101po'a atu",size7:"Pito i l\u0101po'a",defaultFont:"Foliga fou ole mata'itusi",viewSource:"Va'ai 'ile tusitusiga fa'akomepiuta"},tree:{noNodes:"L\u0113ai ni v\u0101ega o maua",noResults:"L\u0113ai ni v\u0101ega na maua"}};})();
