/*!
 * Quasar Framework v2.18.5
 * (c) 2015-present Razvan Stoenescu
 * Released under the MIT License.
 */

(()=>{window.Quasar.Lang.fr={isoName:"fr",nativeName:"Fran\xE7ais",label:{clear:"Effacer",ok:"OK",cancel:"Annuler",close:"Fermer",set:"R\xE9gler",select:"S\xE9lectionner",reset:"R\xE9initialiser",remove:"Supprimer",update:"Mettre \xE0 jour",create:"Cr\xE9er",search:"Rechercher",filter:"Filtrer",refresh:"Rafra\xEEchir",expand:e=>e?`D\xE9velopper "${e}"`:"D\xE9velopper",collapse:e=>e?`R\xE9duire "${e}"`:"Effondrement"},date:{days:"Dimanche_Lundi_Mardi_Mercredi_Jeudi_Vendredi_Samedi".split("_"),daysShort:"Dim_Lun_Mar_Mer_Jeu_Ven_Sam".split("_"),months:"Janvier_F\xE9vrier_Mars_Avril_Mai_Juin_Juillet_Ao\xFBt_Septembre_Octobre_Novembre_D\xE9cembre".split("_"),monthsShort:"Jan_Fev_Mar_Avr_Mai_Juin_Jui_Aou_Sep_Oct_Nov_Dec".split("_"),headerTitle:e=>new Intl.DateTimeFormat("fr",{weekday:"short",day:"numeric",month:"short"}).format(e),firstDayOfWeek:1,format24h:!0,pluralDay:"jours",prevMonth:"Mois pr\xE9c\xE9dent",nextMonth:"Mois suivant",prevYear:"Ann\xE9e pr\xE9c\xE9dente",nextYear:"Ann\xE9e suivante",today:"Aujourd'hui",prevRangeYears:e=>`Pr\xE9c\xE9dent ${e} ann\xE9es`,nextRangeYears:e=>`Suivant ${e} ann\xE9es`},table:{noData:"Aucune donn\xE9e \xE0 afficher",noResults:"Aucune donn\xE9e trouv\xE9e",loading:"Chargement...",selectedRecords:e=>e>0?e+" "+(e===1?"ligne s\xE9lectionn\xE9e":"lignes s\xE9lectionn\xE9es")+".":"Aucune ligne s\xE9lectionn\xE9e.",recordsPerPage:"Lignes par page :",allRows:"Tous",pagination:(e,r,n)=>e+"-"+r+" sur "+n,columns:"Colonnes"},pagination:{first:"Premi\xE8re page",prev:"Page pr\xE9c\xE9dente",next:"Page suivante",last:"Derni\xE8re page"},editor:{url:"URL",bold:"Gras",italic:"Italique",strikethrough:"Barr\xE9",underline:"Soulign\xE9",unorderedList:"Liste non ordonn\xE9e",orderedList:"Liste ordonn\xE9e",subscript:"Indice",superscript:"Exposant",hyperlink:"Hyperlien",toggleFullscreen:"Basculer en plein \xE9cran",quote:"Citation",left:"Aligner \xE0 gauche",center:"Aligner au centre",right:"Aligner \xE0 droite",justify:"Justifier",print:"Imprimer",outdent:"Diminuer l'indentation",indent:"Augmenter l'indentation",removeFormat:"Supprimer la mise en forme",formatting:"Mise en forme",fontSize:"Taille de police",align:"Aligner",hr:"Ins\xE9rer une r\xE8gle horizontale",undo:"Annuler",redo:"Refaire",heading1:"Titre 1",heading2:"Titre 2",heading3:"Titre 3",heading4:"Titre 4",heading5:"Titre 5",heading6:"Titre 6",paragraph:"Paragraphe",code:"Code",size1:"Tr\xE8s petit",size2:"Petit",size3:"Normal",size4:"Moyenne",size5:"Grand",size6:"Tr\xE8s grand",size7:"Maximum",defaultFont:"Police par d\xE9faut",viewSource:"Voir la source"},tree:{noData:"Aucun n\u0153ud \xE0 afficher",noResults:"Aucun n\u0153ud trouv\xE9"}};})();
