/*!
 * Quasar Framework v2.18.5
 * (c) 2015-present Razvan Stoenescu
 * Released under the MIT License.
 */

(()=>{window.Quasar.Lang.tr={isoName:"tr",nativeName:"T\xFCrk\xE7e",label:{clear:"Temizle",ok:"Tamam",cancel:"\u0130ptal",close:"Kapat",set:"Ayarla",select:"Se\xE7",reset:"S\u0131f\u0131rla",remove:"Kald\u0131r",update:"G\xFCncelle",create:"Olu\u015Ftur",search:"Ara",filter:"S\xFCz",refresh:"Yenile",expand:a=>a?`"${a}" geni\u015Flet`:"Geni\u015Fletmek",collapse:a=>a?`"${a}" daralt`:"Y\u0131k\u0131lmak"},date:{days:"Pazar_Pazartesi_Sal\u0131_\xC7ar\u015Famba_Per\u015Fembe_Cuma_Cumartesi".split("_"),daysShort:"Paz_Pzt_Sal_\xC7ar_Per_Cum_Cmt".split("_"),months:"Ocak_\u015Eubat_Mart_Nisan_May\u0131s_Haziran_Temmuz_A\u011Fustos_Eyl\xFCl_Ekim_Kas\u0131m_Aral\u0131k".split("_"),monthsShort:"Oca_\u015Eub_Mar_Nis_May_Haz_Tem_A\u011Fu_Eyl_Eki_Kas_Ara".split("_"),firstDayOfWeek:1,format24h:!0,pluralDay:"g\xFCn",prevMonth:"\xD6nceki ay",nextMonth:"Sonraki ay",prevYear:"\xD6nceki y\u0131l",nextYear:"Sonraki y\u0131l",today:"Bug\xFCn",prevRangeYears:a=>`\xD6nceki ${a} y\u0131l`,nextRangeYears:a=>`Sonraki ${a} y\u0131l`},table:{noData:"Veri yok",noResults:"Uyu\u015Fan kay\u0131t bulunamad\u0131",loading:"Y\xFCkleniyor...",selectedRecords:a=>a+" se\xE7ili kay\u0131t.",recordsPerPage:"Sayfa ba\u015F\u0131na kay\u0131t:",allRows:"T\xFCm\xFC",pagination:(a,e,i)=>a+"-"+e+" toplam "+i,columns:"S\xFCtunlar"},pagination:{first:"\u0130lk sayfa",prev:"\xD6nceki sayfa",next:"Sonraki Sayfa",last:"Son Sayfa"},editor:{url:"URL",bold:"Kal\u0131n",italic:"E\u011Fik",strikethrough:"\xDCst\xFC \xE7izili",underline:"Alt\u0131 \xE7izili",unorderedList:"S\u0131ras\u0131z Liste",orderedList:"S\u0131ral\u0131 Liste",subscript:"Alt betik",superscript:"\xDCst betik",hyperlink:"K\xF6pr\xFC",toggleFullscreen:"Tam ekran\u0131 A\xE7-Kapa",quote:"Al\u0131nt\u0131",left:"Sola hizala",center:"Ortala",right:"Sa\u011Fa hizala",justify:"S\u0131\u011Fd\u0131r",print:"Yazd\u0131r",outdent:"Girintiyi azalt",indent:"Girintiyi art\u0131r",removeFormat:"Bi\xE7imlendirmeyi kald\u0131r",formatting:"Bi\xE7imliyor",fontSize:"Yaz\u0131 Tipi Boyutu",align:"Hizala",hr:"Yatay \xC7izgi Ekle",undo:"Geri Al",redo:"Yinele",heading1:"Ba\u015Fl\u0131k 1",heading2:"Ba\u015Fl\u0131k 2",heading3:"Ba\u015Fl\u0131k 3",heading4:"Ba\u015Fl\u0131k 4",heading5:"Ba\u015Fl\u0131k 5",heading6:"Ba\u015Fl\u0131k 6",paragraph:"Paragraf",code:"Kod",size1:"\xC7ok k\xFC\xE7\xFCk",size2:"K\xFC\xE7\xFCk",size3:"Normal",size4:"Orta-geni\u015F",size5:"B\xFCy\xFCk",size6:"\xC7ok b\xFCy\xFCk",size7:"En b\xFCy\xFCk",defaultFont:"Varsay\u0131lan Yaz\u0131 Tipi",viewSource:"Kayna\u011F\u0131 G\xF6r\xFCnt\xFCle"},tree:{noNodes:"D\xFC\u011F\xFCm yok",noResults:"Uyu\u015Fan d\xFC\u011F\xFCm bulunamad\u0131"}};})();
