/*!
 * Quasar Framework v2.18.5
 * (c) 2015-present Razvan Stoenescu
 * Released under the MIT License.
 */

(()=>{window.Quasar.Lang.sk={isoName:"sk",nativeName:"Sloven\u010Dina",label:{clear:"Vymaza\u0165",ok:"OK",cancel:"Zru\u0161i\u0165",close:"Zavrie\u0165",set:"Nastavi\u0165",select:"Vybra\u0165",reset:"Resetova\u0165",remove:"Odstr\xE1ni\u0165",update:"Upravi\u0165",create:"Vytvori\u0165",search:"H\u013Eada\u0165",filter:"Filtrova\u0165",refresh:"Obnovi\u0165",expand:e=>e?`Rozbali\u0165 "${e}"`:"Rozbali\u0165",collapse:e=>e?`Zbali\u0165 "${e}"`:"Kolaps"},date:{days:"Nede\u013Ea_Pondelok_Utorok_Streda_\u0160tvrtok_Piatok_Sobota".split("_"),daysShort:"Ned_Pon_Uto_Str_\u0160tv_Pia_Sob".split("_"),months:"Janu\xE1r_Febru\xE1r_Marec_Apr\xEDl_M\xE1j_J\xFAn_J\xFAl_August_September_Okt\xF3ber_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_M\xE1j_J\xFAn_J\xFAl_Aug_Sep_Okt_Nov_Dec".split("_"),firstDayOfWeek:1,format24h:!0,pluralDay:"dn\xED",prevMonth:"Predch\xE1dzaj\xFAci mesiac",nextMonth:"Bud\xFAci mesiac",prevYear:"Predch\xE1dzaj\xFAci rok",nextYear:"Bud\xFAci rok",today:"Dnes",prevRangeYears:e=>`Predch\xE1dzaj\xFAce ${e} roky`,nextRangeYears:e=>`\u010Ealej ${e} roky`},table:{noData:"Nie s\xFA dostupn\xE9 \xFAdaje",noResults:"Neboli n\xE1jden\xE9 vyhovuj\xFAce z\xE1znamy",loading:"Na\u010D\xEDtavam...",selectedRecords:e=>e>0?e+" "+(e===1?"riadok vybrat\xFD":e<5?"riadky vybrat\xE9":"riadkov vybrat\xFDch")+".":"\u017Diadne vybrat\xE9 riadky.",recordsPerPage:"Riadkov na str\xE1nku:",allRows:"V\u0161etky",pagination:(e,a,r)=>e+"-"+a+" z "+r,columns:"St\u013Apce"},pagination:{first:"Prv\xE1 str\xE1nka",prev:"Predch\xE1dzaj\xFAca str\xE1nka",next:"\u010Eal\u0161ia str\xE1nka",last:"Posledn\xE1 str\xE1nka"},editor:{url:"URL",bold:"Tu\u010Dn\xE9",italic:"Kurz\xEDva",strikethrough:"Pre\u010Diarknut\xE9",underline:"Pod\u010Diarknut\xE9",unorderedList:"Odr\xE1\u017Eky",orderedList:"\u010C\xEDslovanie",subscript:"Doln\xFD index",superscript:"Horn\xFD index",hyperlink:"Odkaz",toggleFullscreen:"Prepn\xFA\u0165 na cel\xFA obrazovku",quote:"Citova\u0165",left:"Zarovna\u0165 do\u013Eava",center:"Centrova\u0165",right:"Zarovna\u0165 doprava",justify:"Zarovna\u0165 pod\u013Ea okrajov",print:"Tla\u010Di\u0165",outdent:"Zmen\u0161i\u0165 odsadenie",indent:"Zv\xE4\u010D\u0161i\u0165 odsadenie",removeFormat:"Odstr\xE1ni\u0165 form\xE1tovanie",formatting:"Form\xE1tovanie",fontSize:"Ve\u013Ekos\u0165 p\xEDsma",align:"Zarovna\u0165",hr:"Vlo\u017Ei\u0165 horizont\xE1lny oddelova\u010D",undo:"Sp\xE4\u0165",redo:"Znova",heading1:"Hlavi\u010Dka 1",heading2:"Hlavi\u010Dka 2",heading3:"Hlavi\u010Dka 3",heading4:"Hlavi\u010Dka 4",heading5:"Hlavi\u010Dka 5",heading6:"Hlavi\u010Dka 6",paragraph:"Odsek",code:"K\xF3d",size1:"Ve\u013Emi mal\xE9",size2:"Mal\xE9",size3:"Norm\xE1lne",size4:"Stredne ve\u013Ek\xE9",size5:"Ve\u013Ek\xE9",size6:"Ve\u013Emi ve\u013Ek\xE9",size7:"Maxim\xE1lne",defaultFont:"Predvolen\xE9 p\xEDsmo",viewSource:"Zdroj pohladu"},tree:{noNodes:"Nie s\xFA dostupn\xE9 vetvy",noResults:"Neboli n\xE1jden\xE9 vyhovuj\xFAce vetvy"}};})();
