/*!
 * Quasar Framework v2.18.5
 * (c) 2015-present Razvan Stoenescu
 * Released under the MIT License.
 */

(()=>{window.Quasar.Lang.vi={isoName:"vi",nativeName:"Ti\u1EBFng Vi\u1EC7t",label:{clear:"X\xF3a h\u1EBFt",ok:"OK",cancel:"H\u1EE7y",close:"\u0110\xF3ng",set:"Thi\u1EBFt \u0111\u1EB7t",select:"Ch\u1ECDn",reset:"\u0110\u1EB7t l\u1EA1i",remove:"G\u1EE1 b\u1ECF",update:"C\u1EADp nh\u1EADt",create:"T\u1EA1o",search:"T\xECm ki\u1EBFm",filter:"B\u1ED9 l\u1ECDc",refresh:"L\xE0m m\u1EDBi",expand:n=>n?`M\u1EDF r\u1ED9ng "${n}"`:"M\u1EDF r\u1ED9ng",collapse:n=>n?`Thu g\u1ECDn "${n}"`:"S\u1EE5p \u0111\u1ED5"},date:{days:"Ch\u1EE7 Nh\u1EADt_Th\u1EE9 Hai_Th\u1EE9 Ba_Th\u1EE9 T\u01B0_Th\u1EE9 N\u0103m_Th\u1EE9 S\xE1u_Th\u1EE9 B\u1EA3y".split("_"),daysShort:"CN_T2_T3_T4_T5_T6_T7".split("_"),months:"Th\xE1ng M\u1ED9t_Th\xE1ng Hai_Th\xE1ng Ba_Th\xE1ng T\u01B0_Th\xE1ng N\u0103m_Th\xE1ng S\xE1u_Th\xE1ng B\u1EA3y_Th\xE1ng T\xE1m_Th\xE1ng Ch\xEDn_Th\xE1ng M\u01B0\u1EDDi_Th\xE1ng M\u01B0\u1EDDi M\u1ED9t_Th\xE1ng M\u01B0\u1EDDi Hai".split("_"),monthsShort:"Th1_Th2_Th3_Th4_Th5_Th6_Th7_Th8_Th9_Th10_Th11_Th12".split("_"),firstDayOfWeek:1,format24h:!1,pluralDay:"ng\xE0y",prevMonth:"Th\xE1ng tr\u01B0\u1EDBc",nextMonth:"Th\xE1ng sau",prevYear:"N\u0103m tr\u01B0\u1EDBc",nextYear:"N\u0103m sau",today:"H\xF4m nay",prevRangeYears:n=>`Tr\u01B0\u1EDBc ${n} n\u0103m`,nextRangeYears:n=>`Ti\u1EBFp theo ${n} n\u0103m`},table:{noData:"Kh\xF4ng c\xF3 d\u1EEF li\u1EC7u",noResults:"Kh\xF4ng t\xECm th\u1EA5y k\u1EBFt qu\u1EA3",loading:"\u0110ang t\u1EA3i",selectedRecords:n=>n===1?"1 h\xE0ng \u0111\xE3 ch\u1ECDn.":(n===0?"Kh\xF4ng c\xF3 h\xE0ng n\xE0o":n)+" h\xE0ng \u0111\xE3 ch\u1ECDn.",recordsPerPage:"H\xE0ng tr\xEAn m\u1ED7i trang:",allRows:"T\u1EA5t c\u1EA3",pagination:(n,h,t)=>n+"-"+h+" c\u1EE7a "+t,columns:"C\u1ED9t"},pagination:{first:"Trang \u0111\u1EA7u ti\xEAn",prev:"Trang tr\u01B0\u1EDBc",next:"Trang ti\u1EBFp theo",last:"Trang cu\u1ED1i c\xF9ng"},editor:{url:"URL",bold:"\u0110\u1EADm",italic:"Nghi\xEAng",strikethrough:"G\u1EA1ch gi\u1EEFa",underline:"G\u1EA1ch d\u01B0\u1EDBi",unorderedList:"Danh s\xE1ch kh\xF4ng theo th\u1EE9 t\u1EF1",orderedList:"Danh s\xE1ch theo th\u1EE9 t\u1EF1",subscript:"Ch\u1EC9 s\u1ED1 d\u01B0\u1EDBi",superscript:"Ch\u1EC9 s\u1ED1 tr\xEAn",hyperlink:"Li\xEAn k\u1EBFt",toggleFullscreen:"\u0110i\u1EC1u ch\u1EC9nh ch\u1EBF \u0111\u1ED9 to\xE0n m\xE0n h\xECnh",quote:"Tr\xEDch d\u1EABn",left:"C\u0103n tr\xE1i",center:"C\u0103n gi\u1EEFa",right:"C\u0103n ph\u1EA3i",justify:"C\u0103n \u0111\u1EC1u 2 b\xEAn",print:"In",outdent:"Gi\u1EA3m l\u1EC1",indent:"T\u0103ng l\u1EC1",removeFormat:"X\xF3a \u0111\u1ECBnh d\u1EA1ng",formatting:"\u0110\u1ECBnh d\u1EA1ng",fontSize:"K\xEDch c\u1EE1 ph\xF4ng",align:"C\u0103n ch\u1EC9nh",hr:"Ch\xE8n Quy T\u1EAFc Ngang",undo:"Ho\xE0n t\xE1c",redo:"L\xE0m l\u1EA1i",heading1:"Ti\xEAu \u0111\u1EC1 1",heading2:"Ti\xEAu \u0111\u1EC1 2",heading3:"Ti\xEAu \u0111\u1EC1 3",heading4:"Ti\xEAu \u0111\u1EC1 4",heading5:"Ti\xEAu \u0111\u1EC1 5",heading6:"Ti\xEAu \u0111\u1EC1 6",paragraph:"\u0110o\u1EA1n",code:"M\xE3",size1:"R\u1EA5t nh\u1ECF",size2:"Nh\u1ECF v\u1EEBa",size3:"Th\u01B0\u1EDDng",size4:"To v\u1EEBa",size5:"To",size6:"R\u1EA5t To",size7:"T\u1ED1i \u0111a",defaultFont:"Ph\xF4ng m\u1EB7c \u0111\u1ECBnh",viewSource:"Xem ngu\u1ED3n"},tree:{noNodes:"Kh\xF4ng c\xF3 n\u1ED1t n\xE0o c\xF3 s\u1EB5n",noResults:"Kh\xF4ng t\xECm th\u1EA5y c\xE1c n\u1ED1t"}};})();
