/*!
 * Quasar Framework v2.18.5
 * (c) 2015-present Razvan Stoenescu
 * Released under the MIT License.
 */

(()=>{window.Quasar.Lang.is={isoName:"is",nativeName:"\xCDslenska",label:{clear:"Hreinsa",ok:"\xD3kei",cancel:"H\xE6tta vi\xF0",close:"Loka",set:"Setja",select:"Velja",reset:"Endurstilla",remove:"Fjarl\xE6gja",update:"Uppf\xE6ra",create:"B\xFAa til",search:"Leita",filter:"S\xEDa",refresh:"Endurhla\xF0a",expand:r=>r?`St\xE6kka "${r}"`:"St\xE6kka\xF0u",collapse:r=>r?`Draga saman "${r}"`:"Hrun"},date:{days:"Sunnudagur_M\xE1nudagur_\xDEri\xF0judagur_Mi\xF0vikudagur_Fimmtudagur_F\xF6studagur_Laugardagur".split("_"),daysShort:"Sun_M\xE1n_\xDEri_Mi\xF0_Fim_F\xF6s_Lau".split("_"),months:"Jan\xFAar_Febr\xFAar_Mars_Apr\xEDl_Ma\xED_J\xFAn\xED_J\xFAl\xED_\xC1g\xFAst_September_Okt\xF3ber_N\xF3vember_Desember".split("_"),monthsShort:"Jan_Feb_Mar_Apr_Ma\xED_J\xFAn_J\xFAl_\xC1g\xFA_Sep_Okt_N\xF3v_Des".split("_"),firstDayOfWeek:0,format24h:!0,pluralDay:"dagar",prevMonth:"Fyrri m\xE1nu\xF0ur",nextMonth:"N\xE6sta m\xE1nu\xF0",prevYear:"Fyrra \xE1r",nextYear:"\xC1 n\xE6sta \xE1ri",today:"\xCD dag",prevRangeYears:r=>`Fyrri ${r} \xE1r`,nextRangeYears:r=>`N\xE6st ${r} \xE1r`},table:{noData:"Engin g\xF6gn tilt\xE6k",noResults:"Engar samsvarandi skr\xE1r fundust",loading:"Augnablik...",selectedRecords:r=>r===1?"1 f\xE6rsla valin.":(r===0?"Engar":r)+" f\xE6rslur valdar.",recordsPerPage:"F\xE6rslur \xE1 hverri s\xED\xF0u:",allRows:"Allar",pagination:(r,a,e)=>r+"-"+a+" af "+e,columns:"D\xE1lkar"},pagination:{first:"Fyrsta bla\xF0s\xED\xF0a",prev:"Fyrri bla\xF0s\xED\xF0a",next:"N\xE6sta bla\xF0s\xED\xF0a",last:"S\xED\xF0asta bla\xF0s\xED\xF0a"},editor:{url:"Sl\xF3\xF0",bold:"Feitletra",italic:"Sk\xE1letra",strikethrough:"Strika \xED gegnum",underline:"Undirstrika",unorderedList:"\xD3ra\xF0a\xF0ur listi",orderedList:"Ra\xF0a\xF0ur listi",subscript:"Hn\xE9letur",superscript:"Brj\xF3stletur",hyperlink:"Hlekkur",toggleFullscreen:"Fullskj\xE1r af/\xE1",quote:"Tilvitnun",left:"Vinstrijafna",center:"Mi\xF0jujafna",right:"H\xE6grijafna",justify:"Jafna",print:"Prenta",outdent:"Minnka inndr\xE1tt",indent:"Auka inndr\xE1tt",removeFormat:"Hreinsa sni\xF0",formatting:"Sni\xF0",fontSize:"Leturst\xE6r\xF0",align:"Jafna",hr:"L\xE1r\xE9tt l\xEDna",undo:"Afturkalla",redo:"Endurgera",heading1:"Fyrirs\xF6gn 1",heading2:"Fyrirs\xF6gn 2",heading3:"Fyrirs\xF6gn 3",heading4:"Fyrirs\xF6gn 4",heading5:"Fyrirs\xF6gn 5",heading6:"Fyrirs\xF6gn 6",paragraph:"Efnisgrein",code:"K\xF3\xF0i",size1:"Mj\xF6g l\xEDtill",size2:"D\xE1l\xEDti\xF0 l\xEDtill",size3:"Venjulegur",size4:"Mi\xF0lungsst\xF3r",size5:"St\xF3r",size6:"Mj\xF6g st\xF3r",size7:"Risast\xF3r",defaultFont:"Sj\xE1lfgefi\xF0 letur",viewSource:"Sj\xE1 k\xF3\xF0a"},tree:{noNodes:"Engar n\xF3\xF0ur \xED bo\xF0i",noResults:"Engar samsvarandi n\xF3\xF0ur fundust"}};})();
