/*!
 * Quasar Framework v2.18.5
 * (c) 2015-present Razvan Stoenescu
 * Released under the MIT License.
 */

(()=>{window.Quasar.Lang.sq={isoName:"sq",nativeName:"Shqip",label:{clear:"Pastro",ok:"OK",cancel:"Anulo",close:"Mbyll",set:"Vendos",select:"Zgjidh",reset:"Rivendos",remove:"Hiq",update:"P\xEBrdit\xEBso",create:"Krijo",search:"K\xEBrko",filter:"Filtro",refresh:"Rifresko",expand:e=>e?`Zgjero "${e}"`:"Zgjero",collapse:e=>e?`Tkur "${e}"`:"Tkur"},date:{days:"E diel_E h\xEBn\xEB_E mart\xEB_E m\xEBrkur\xEB_E enjte_E premte_E shtun\xEB".split("_"),daysShort:"Die_H\xEBn_Mar_M\xEBr_Enj_Pre_Sht".split("_"),months:"Janar_Shkurt_Mars_Pril_Maj_Qershor_Korrik_Gusht_Shtator_Tetor_N\xEBntor_Dhjetor".split("_"),monthsShort:"Jan_Shk_Mar_Pri_Maj_Qer_Kor_Gus_Sht_Tet_N\xEBn_Dhj".split("_"),firstDayOfWeek:1,format24h:!0,pluralDay:"dit\xEB",prevMonth:"Muaji i kaluar",nextMonth:"Muaji i ardhsh\xEBm",prevYear:"Viti i kaluar",nextYear:"Viti i ardhsh\xEBm",today:"Sot",prevRangeYears:e=>`Vitet e m\xEBparshme ${e}`,nextRangeYears:e=>`Vitet e ardhshme ${e}`},table:{noData:"Nuk ka t\xEB dh\xEBna",noResults:"Nuk u gjet\xEBn rezultate",loading:"Duke ngarkuar...",selectedRecords:e=>e===1?"1 regjistrim i p\xEBrzgjedhur.":(e===0?"Asnj\xEB":e)+" regjistrime t\xEB p\xEBrzgjedhura.",recordsPerPage:"Regjistrime p\xEBr faqe:",allRows:"T\xEB gjitha",pagination:(e,r,i)=>`${e}-${r} nga ${i}`,columns:"Kolonat"},pagination:{first:"Faqja e par\xEB",prev:"Faqja e m\xEBparshme",next:"Faqja tjet\xEBr",last:"Faqja e fundit"},editor:{url:"URL",bold:"Trash",italic:"Pjerr\xEBt",strikethrough:"Vij\xEB n\xEB mes",underline:"N\xEBnvizuar",unorderedList:"List\xEB pa rend",orderedList:"List\xEB e renditur",subscript:"N\xEBnshkrim",superscript:"Supershkrim",hyperlink:"Lidhje",toggleFullscreen:"Ndrysho ekranin e plot\xEB",quote:"Citim",left:"Rreshto majtas",center:"Rreshto n\xEB qend\xEBr",right:"Rreshto djathtas",justify:"Justifiko",print:"Printo",outdent:"Zvog\xEBlo indentimin",indent:"Rrit indentimin",removeFormat:"Hiq formatimin",formatting:"Formatimi",fontSize:"Madh\xEBsia e shkronjave",align:"Rreshtimi",hr:"Shto vij\xEB horizontale",undo:"Zhb\xEBj",redo:"Rib\xEBj",heading1:"Titulli 1",heading2:"Titulli 2",heading3:"Titulli 3",heading4:"Titulli 4",heading5:"Titulli 5",heading6:"Titulli 6",paragraph:"Paragraf",code:"Kod",size1:"Shum\xEB i vog\xEBl",size2:"Pak i vog\xEBl",size3:"Normal",size4:"Mesatar",size5:"I madh",size6:"Shum\xEB i madh",size7:"Maksimal",defaultFont:"Shkronja e parazgjedhur",viewSource:"Shiko burimin"},tree:{noNodes:"Nuk ka nyje t\xEB disponueshme",noResults:"Nuk u gjet\xEBn nyje p\xEBrputh\xEBse"}};})();
