/*!
 * Quasar Framework v2.18.5
 * (c) 2015-present Razvan Stoenescu
 * Released under the MIT License.
 */

(()=>{window.Quasar.Lang.ro={isoName:"ro",nativeName:"Rom\xE2n\u0103",label:{clear:"Gole\u0219te",ok:"OK",cancel:"Anuleaz\u0103",close:"\xCEnchide",set:"Seteaz\u0103",select:"Alege",reset:"Reseteaz\u0103",remove:"Elimin\u0103",update:"Actualizeaz\u0103",create:"Creeaz\u0103",search:"Caut\u0103",filter:"Filtreaz\u0103",refresh:"Actualizeaz\u0103",expand:e=>e?`Deschide\u021Bi "${e}"`:"Deschide",collapse:e=>e?`\xCEnchide "${e}"`:"\xCEnchide"},date:{days:"Duminic\u0103_Luni_Mar\u021Bi_Miercuri_Joi_Vineri_S\xE2mb\u0103t\u0103".split("_"),daysShort:"Dum_Lun_Mar_Mie_Joi_Vin_S\xE2m".split("_"),months:"Ianuarie_Februarie_Martie_Aprilie_Mai_Iunie_Iulie_August_Septembrie_Octombrie_Noiembrie_Decembrie".split("_"),monthsShort:"Ian_Feb_Mar_Apr_Mai_Iun_Iul_Aug_Sep_Oct_Nov_Dec".split("_"),firstDayOfWeek:1,format24h:!0,pluralDay:"zile",prevMonth:"Luna anterioar\u0103",nextMonth:"Luna viitoare",prevYear:"Anul precedent",nextYear:"Anul viitor",today:"Ast\u0103zi",prevRangeYears:e=>`Anterior ${e} ani`,nextRangeYears:e=>`Urm\u0103torul ${e} ani`},table:{noData:"Nu sunt date disponibile",noResults:"Nu am g\u0103sit \xEEnregistr\u0103ri care s\u0103 corespund\u0103",loading:"Se \xEEncarc\u0103...",selectedRecords:e=>e>1?e+" \xEEnregistr\u0103ri selectate.":(e===0?"Nici o":"1")+" \xEEnregistrare selectat\u0103.",recordsPerPage:"\xCEnregistr\u0103ri pe pagin\u0103:",allRows:"Toate",pagination:(e,i,a)=>e+"-"+i+" din "+a,columns:"Coloane"},pagination:{first:"Prima pagin\u0103",prev:"Pagina anterioar\u0103",next:"Pagina urm\u0103toare",last:"Ultima pagin\u0103"},editor:{url:"URL",bold:"\xCEngro\u0219at",italic:"\xCEnclinat",strikethrough:"T\u0103iat",underline:"Subliniat",unorderedList:"List\u0103 neordonat\u0103",orderedList:"List\u0103 ordonat\u0103",subscript:"Dedesubt",superscript:"Deasupra",hyperlink:"Hyperlink",toggleFullscreen:"Comut\u0103 ecran complet",quote:"Citat",left:"Aliniere la st\xE2nga",center:"Aliniere la centru",right:"Aliniere la dreapta",justify:"Aliniere total\u0103",print:"Tip\u0103re\u0219te",outdent:"Scade spa\u021Bierea",indent:"Cre\u0219te spa\u021Bierea",removeFormat:"\xCEndep\u0103rteaz\u0103 format\u0103rile",formatting:"Formatare",fontSize:"M\u0103rime font",align:"Aliniaz\u0103",hr:"Adaug\u0103 linie orizontal\u0103",undo:"Schimb\u0103 inapoi",redo:"Ref\u0103",heading1:"Heading 1",heading2:"Heading 2",heading3:"Heading 3",heading4:"Heading 4",heading5:"Heading 5",heading6:"Heading 6",paragraph:"Paragraf",code:"Cod",size1:"Foarte mic",size2:"Mic",size3:"Normal",size4:"Mediu-mare",size5:"Big",size6:"Foarte mare",size7:"Maxim",defaultFont:"Font implicit",viewSource:"Vizualizare surs\u0103"},tree:{noNodes:"Nu sunt date disponibile",noResults:"Nu am g\u0103sit noduri care s\u0103 corespund\u0103"}};})();
