/*!
 * Quasar Framework v2.18.5
 * (c) 2015-present Razvan Stoenescu
 * Released under the MIT License.
 */

(()=>{window.Quasar.Lang.ca={isoName:"ca",nativeName:"Catal\xE0",label:{clear:"Netejar",ok:"D'acord",cancel:"Cancel\xB7lar",close:"Tancar",set:"Definir",select:"Seleccionar",reset:"Reinicialitzar",remove:"Suprimir",update:"Actualitzar",create:"Crear",search:"Cercar",filter:"Filtrar",refresh:"Refrescar",expand:e=>e?`Amplia "${e}"`:"Expandir",collapse:e=>e?`Col\xB7lapse "${e}"`:"Col\xB7lapse"},date:{days:"Diumenge_Dilluns_Dimarts_Dimecres_Dijous_Divendres_Dissabte".split("_"),daysShort:"Dg_Dl_Dt_Dc_Dj_Dv_Ds".split("_"),months:"Gener_Febrer_Mar\xE7_Abril_Maig_Juny_Juliol_Agost_Setembre_Octubre_Novembre_Desembre".split("_"),monthsShort:"Gen_Feb_Mar_Abr_Mai_Jun_Jul_Ago_Set_Oct_Nov_Des".split("_"),firstDayOfWeek:1,format24h:!0,pluralDay:"dies",prevMonth:"Mes anterior",nextMonth:"El mes vinent",prevYear:"Any anterior",nextYear:"L'any vinent",today:"Avui",prevRangeYears:e=>`Anterior ${e} anys`,nextRangeYears:e=>`A continuaci\xF3 ${e} anys`},table:{noData:"No hi ha dades disponibles",noResults:"No s'han trobat resultats",loading:"Carregant...",selectedRecords:e=>e>1?e+" files seleccionades.":(e===0?"Sense":"1")+" fila seleccionada.",recordsPerPage:"Files per p\xE0gina:",allRows:"Totes",pagination:(e,a,r)=>e+"-"+a+" de "+r,columns:"Columnes"},pagination:{first:"Primera p\xE0gina",prev:"P\xE0gina anterior",next:"P\xE0gina seg\xFCent",last:"\xDAltima p\xE0gina"},editor:{url:"URL",bold:"Negreta",italic:"Cursiva",strikethrough:"Barrat",underline:"Subratllat",unorderedList:"Llista sense ordre",orderedList:"Llista amb ordre",subscript:"Sub\xEDndex",superscript:"Super\xEDndex",hyperlink:"Hiperenlla\xE7",toggleFullscreen:"Commutar pantalla completa",quote:"Cita",left:"Alinear a l'esquerra",center:"Alinear al centre",right:"Alinear a la dreta",justify:"Alinear justificat",print:"Imprimir",outdent:"Augmentar identaci\xF3",indent:"Disminuir identaci\xF3",removeFormat:"Llevar el format",formatting:"Formatant",fontSize:"Mida de la font",align:"Alinear",hr:"Inserir una l\xEDnea horitzontal",undo:"Desfer",redo:"Refer",heading1:"Encap\xE7alament 1",heading2:"Encap\xE7alament 2",heading3:"Encap\xE7alament 3",heading4:"Encap\xE7alament 4",heading5:"Encap\xE7alament 5",heading6:"Encap\xE7alament 6",paragraph:"Par\xE0graf",code:"Codi",size1:"Molt petit",size2:"Petit",size3:"Normal",size4:"Mitj\xE0",size5:"Gran",size6:"Molt gran",size7:"M\xE0xim",defaultFont:"Font per defecte",viewSource:"Veure font"},tree:{noNodes:"No hi ha nodes disponibles",noResults:"No s'han trobat nodes"}};})();
