/*!
 * Quasar Framework v2.18.5
 * (c) 2015-present Razvan Stoenescu
 * Released under the MIT License.
 */

(()=>{window.Quasar.Lang.lu={isoName:"lu",nativeName:"Kiluba",label:{clear:"Eidel",ok:"OK",cancel:"Oofbriechen",close:"Schl\xE9issen",set:"Setzen",select:"Auswielen",reset:"Zer\xE9cksetzen",remove:"L\xE4schen",update:"Aktualis\xE9ieren",create:"Erstellen",search:"Sichen",filter:"Filter",refresh:"Aktualis\xE9ieren",expand:e=>e?`"${e}" erweideren`:"Erweideren",collapse:e=>e?`"${e}" zesummeklappen`:"Zesummebroch"},date:{days:"Sonndeg_M\xE9indeg_D\xEBnschdeg_M\xEBttwoch_Donneschdeg_Freideg_Samschdeg".split("_"),daysShort:"So_M\xE9_D\xEB_M\xEB_Do_Fr_Sa".split("_"),months:"Januar_Februar_M\xE4rz_Abr\xEBll_M\xE4i_Juni_Juli_August_September_Oktober_November_Dezember".split("_"),monthsShort:"Jan_Feb_M\xE4r_Abr_M\xE4i_Jun_Jul_Aug_Sep_Okt_Nov_Dec".split("_"),firstDayOfWeek:1,format24h:!0,pluralDay:"deeg",prevMonth:"Virdrun Mount",nextMonth:"N\xE4chste Mount",prevYear:"Joer virdrun",nextYear:"N\xE4chst Joer",today:"Haut",prevRangeYears:e=>`Pr\xE9visch ${e} Joren`,nextRangeYears:e=>`N\xE4chst ${e} Joren`},table:{noData:"Keng Donn\xE9en\xA0do",noResults:"Keng Antr\xE4g fonnt",loading:"Luedt...",selectedRecords:e=>e===1?"1 ausgewielten Zeil.":(e===0?"Keng":e)+" Zeilen ausgewielt.",recordsPerPage:"Zeilen pro S\xE4it:",allRows:"All",pagination:(e,r,n)=>e+"-"+r+" vun "+n,columns:"Kolonnen"},pagination:{first:"\xC9ischt S\xE4it",prev:"Virdrun S\xE4it",next:"N\xE4chst S\xE4it",last:"Lescht S\xE4it"},editor:{url:"URL",bold:"Fett",italic:"Kursiv",strikethrough:"Duerchgestrach",underline:"\xCBnnerstrach",unorderedList:"Ongeuerdnet L\xEBscht",orderedList:"Geuerdnet L\xEBscht",subscript:"d\xE9ifgestallt",superscript:"h\xE9ichgestallt",hyperlink:"Link",toggleFullscreen:"Vollbild \xEBmschalten",quote:"Zitat",left:"lenksb\xFCndeg",center:"zentr\xE9iert",right:"riedsb\xFCndeg",justify:"Ausriichten",print:"Drucken",outdent:"ausr\xEBcken",indent:"anr\xEBcken",removeFormat:"Format\xE9ierung l\xF6schen",formatting:"Format\xE9iere",fontSize:"Schr\xEBftgr\xE9isst",align:"Ausriichten",hr:"Horizontal Linn ans\xEBtzen",undo:"R\xE9ckg\xE4ngeg",redo:"Restaur\xE9ieren",heading1:"Iwwerschr\xEBft 1",heading2:"Iwwerschr\xEBft 2",heading3:"Iwwerschr\xEBft 3",heading4:"Iwwerschr\xEBft 4",heading5:"Iwwerschr\xEBft 5",heading6:"Iwwerschr\xEBft 6",paragraph:"Paragraphe",code:"Code",size1:"Ganz kleng",size2:"E b\xEBssi kleng",size3:"Normal",size4:"Grouss",size5:"Gr\xE9isser",size6:"Ganz grouss",size7:"Maximum",defaultFont:"Standard Schr\xEBft",viewSource:"Umthombo wokubuka"},tree:{noNodes:"Keng Kniet verf\xFCgbar",noResults:"Keng passend Kniet fonnt"}};})();
