/*!
 * Quasar Framework v2.18.5
 * (c) 2015-present Razvan Stoenescu
 * Released under the MIT License.
 */

(()=>{window.Quasar.Lang.nbNO={isoName:"nb-NO",nativeName:"Norsk",label:{clear:"T\xF8m",ok:"OK",cancel:"Avbryt",close:"Lukk",set:"Bruk",select:"Velg",reset:"Nullstill",remove:"Slett",update:"Oppdater",create:"Lag",search:"S\xF8k",filter:"Filter",refresh:"Oppdater",expand:e=>e?`Utvid "${e}"`:"Utvide",collapse:e=>e?`Skjul "${e}"`:"Kollapse"},date:{days:"S\xF8ndag_Mandag_Tirsdag_Onsdag_Torsdag_Fredag_L\xF8rdag".split("_"),daysShort:"S\xF8n_Man_Tir_Ons_Tor_Fre_L\xF8r".split("_"),months:"Januar_Februar_Mars_April_Mai_Juni_Juli_August_September_Oktober_November_Desember".split("_"),monthsShort:"Jan_Feb_Mar_Apr_Mai_Jun_Jul_Aug_Sep_Okt_Nov_Des".split("_"),firstDayOfWeek:1,format24h:!0,pluralDay:"dager",prevMonth:"Forrige m\xE5ned",nextMonth:"Neste m\xE5ned",prevYear:"Forrige \xE5r",nextYear:"Neste \xE5r",today:"I dag",prevRangeYears:e=>`Forrige ${e} \xE5r`,nextRangeYears:e=>`Neste ${e} \xE5r`},table:{noData:"Ingen data tilgjengelig",noResults:"Ingen treff i data funnet",loading:"Laster...",row:"rad",selectedRecords:e=>e>0?e+" row"+(e===1?"":"s")+" valgt.":"Ingen valgte rader.",recordsPerPage:"Rader pr side:",allRows:"Alle",pagination:(e,r,t)=>e+"-"+r+" av "+t,columns:"Kolonner"},pagination:{first:"F\xF8rste side",prev:"Forrige side",next:"Neste side",last:"Siste side"},editor:{url:"URL",bold:"Fet",italic:"Kursiv",strikethrough:"Gjennomstreking",underline:"Understrek",unorderedList:"Uordnet liste",orderedList:"Ordnet liste",subscript:"Senket skrift",superscript:"Hevet skrift",hyperlink:"Lenke",toggleFullscreen:"Av/p\xE5 fullskjerm",quote:"Sitat",left:"Venstrestill",center:"Sentrer",right:"H\xF8yrestill",justify:"Tilpasset bredde",print:"Skriv ut",outdent:"Midre innrykk",indent:"St\xF8rre innrykk",removeFormat:"Fjern formatering",formatting:"Formatering",fontSize:"Fontst\xF8rrelse",align:"Stilling",hr:"Sett inn horisontal linje",undo:"Angre",redo:"Gjenta",heading1:"Overskrift 1",heading2:"Overskrift 2",heading3:"Overskrift 3",heading4:"Overskrift 4",heading5:"Overskrift 5",heading6:"Overskrift 6",paragraph:"Avsnitt",code:"Kode",size1:"Veldig liten",size2:"Liten",size3:"Normal",size4:"Medium-stor",size5:"Stor",size6:"Veldig stor",size7:"Maximum",defaultFont:"Normal font",viewSource:"Se kilde"},tree:{noNodes:"Ingen noder tilgjengelig",noResults:"Ingen treff i noder funnet"}};})();
